package examples.proxy;

import java.util.*;
import javax.sip.*;
import javax.sip.message.*;
import javax.sip.header.*;
import javax.sip.address.*;
import examples.proxy.registrar.*;
import java.text.ParseException;
import java.io.IOException;
import examples.proxy.authentication.*;
import examples.proxy.presenceserver.*;

/** Proxy Entry point.
 *
 *@version  JAIN-SIP-1.1
 *
 *@author Olivier Deruelle <deruelle@nist.gov>  (primary) <br/>
 * M. Ranganathan <mranga@nist.gov> (bug fixes and hacks) <br/>
 *<a href="{@docRoot}/uncopyright.html">This code is in the public domain.</a>
 *
 */
public class Proxy implements SipListener  {
    
    // Map the server transactions with the client transactions
    protected TransactionsMapping transactionsMapping;
    protected SipStack sipStack;
    protected SipProvider sipProvider;
    
    protected MessageFactory messageFactory;
    protected HeaderFactory headerFactory;
    protected AddressFactory addressFactory;
    
    protected Configuration configuration;
    protected PresenceServer presenceServer;
  
    protected Registrar registrar;
    protected ProxyUtilities proxyUtilities;
    protected Authentication authentication;
   
    protected String stackDomain;
    
    public String getStackDomain() {
        return stackDomain;
    }
    
    public AddressFactory getAddressFactory() {
        return addressFactory;
    }
    
    public MessageFactory getMessageFactory() {
        return messageFactory;
    }
    
    public HeaderFactory getHeaderFactory() {
        return headerFactory;
    }
    
    public Registrar getRegistrar() {
        return registrar;
    }
    
    public TransactionsMapping getTransactionsMapping() {
        return transactionsMapping;
    }
    
    
    public boolean isPresenceServer() {
        return configuration.enablePresenceServer;
    }
    
    
    public PresenceServer getPresenceServer() {
            return presenceServer;
    }
    
    
    
    
    public ProxyUtilities getProxyUtilities() {
        return proxyUtilities;
    }
    
    public SipStack getSipStack() {
        return sipStack;
    }
    
    public SipProvider getSipProvider() {
        return sipProvider;
    }
    
    public Configuration getConfiguration() {
        return configuration;
    }
    
    
    /** Creates new Proxy */
    public Proxy(String confFile) throws Exception{
        this.transactionsMapping = new TransactionsMapping();
        if (confFile==null) {
            System.out.println
            ("ERROR: Set the configuration file flag: " +
            "USE: -cf configuration_file_location.xml"  );
        }
        else {
            try {
               
                // First, let's parse the configuration file.
                ProxyConfigurationHandler handler=
                new ProxyConfigurationHandler(confFile);
                configuration=handler.getConfiguration();
                if (configuration==null ||
                !configuration.isValidConfiguration()) {
                    System.out.println
                    ("ERROR: the configuration file is not correct!"+
                    " Correct the errors first.");
                    throw new Exception
		    ("ERROR: the configuration file is not correct!"+
                    " Correct the errors first.");
                }
                else {
                    configuration.initMethodsProcessing();
                    
                
                    proxyUtilities=new ProxyUtilities(this);
                    presenceServer=new PresenceServer();
                    registrar=new Registrar(this);
                }
            }
            catch (Exception ex) {
                System.out.println
                ("ERROR: exception raised while initializing the proxy");
                ex.printStackTrace();
                throw new Exception
		("ERROR: exception raised while initializing the proxy");
            }
        }
    }
    
   
   
    /** This is a listener method.
     */ 
    public void processRequest(RequestEvent requestEvent) {
        try {
            Request request = requestEvent.getRequest();
            request=(Request)request.clone(); 
            ProxyDebug.println
            ("\n****************************************************"+
            "\nRequest " + request.getMethod() +" received.\n");
            
             // Looping detection:
            if ( proxyUtilities.hasViaHeaderProxy(sipStack,request) ) {
                ProxyDebug.println
		    	("ProcessRequest: Loop detected, we drop the request");
                return;
            }
            else {
                  ProxyDebug.println
		    	("ProcessRequest: No Loop detected...");
            }
            
            
            ServerTransaction serverTransaction =
            requestEvent.getServerTransaction();
            if (serverTransaction == null) {
		// Forward statelessly to the next hop.
		if (ProxyDebug.debug) 
		   ProxyDebug.println("The server transaction does not exist, we forward"+
                   " the request statelessly: " +
				request);
		Request newRequest = proxyUtilities.addProxyViaHeader
			(request,"udp",null);
		sipProvider.sendStatelessRequest(request);
		return;
	    }
            else {
                 ProxyDebug.println("The server transaction exists, here is the "+
                 " server transaction: "+serverTransaction);
            }
            
           
            if (serverTransaction.getDialog()!=null) {
                ProxyDebug.println("The dialog (isServer:"+serverTransaction.getDialog().isServer()+
                ") for the server transaction exists,"+
                " here is the dialog Identifier: "+serverTransaction.getDialog().getDialogId());
            }
            else {
                ProxyDebug.println("The dialog for the server transaction does not exist");
            }
            
        
            // Authentication process:
            if (configuration.enableAuthentication) {
                if (!authentication.isAuthorized(request) ) {
                        Response response=authentication.getResponse(request);
                        if (response!=null)
                        sipProvider.sendResponse(serverTransaction,response);
                        ProxyDebug.println
		    	("processRequest: 407 Proxy Authorization "
			 + "Required replied:\n"+response);
                        return;
                }
            }
            
           
            
            // Max forwards:
            MaxForwardsHeader mf = (MaxForwardsHeader) request.getHeader
                (MaxForwardsHeader.NAME);
            if (mf == null) {
                // MaxForwards header:
                MaxForwardsHeader maxForwardsHeader=headerFactory.createMaxForwardsHeader(70);
                request.addHeader(maxForwardsHeader); 
            }
            else 
            if ( mf.getMaxForwards() == 0) {
                Response response = messageFactory.createResponse
                (Response.TOO_MANY_HOPS,request);
                sipProvider.sendResponse(serverTransaction,response);
                if (ProxyDebug.debug)
                    ProxyDebug.println
                    ("processRequest: TOO_MANY_HOPS  replied");
                    return;
            } 
            else {
                // Decrement max forwards
                // by 1 to avoid endless loobpack.
                mf.setMaxForwards(mf.getMaxForwards() - 1);
            }
            
            
            MethodProcessing methodProcessing=(MethodProcessing)configuration.
            getMethodProcessing(request.getMethod());
            if (methodProcessing==null) {
                ProxyDebug.println
                ("Proxy, Class processing the method is not found, " +
                " we use the default"+
                " method processing");
                methodProcessing=(MethodProcessing)configuration.
                getMethodProcessing("DEFAULT_PROCESSING");
                if (methodProcessing==null) {
                    ProxyDebug.println
                    ("Proxy, Default Class processing not found, " +
                    " we drop the request");
                }
                else {
                    methodProcessing.processRequest(this,request,serverTransaction);
                }
            }
            else {
                methodProcessing.processRequest(this,request,serverTransaction);
            }
            ProxyDebug.println
            ("\n****************************************************");
            
        }
        catch (Exception ex){
            ProxyDebug.println("Proxy, Unable to process the request:");
            ex.printStackTrace();
        }
    }
    
    
    
    /** This is a listener method.
     */
    
    public void processResponse(ResponseEvent responseEvent) {
        try{
            
            Response response = responseEvent.getResponse();
            ClientTransaction clientTransaction =
            responseEvent.getClientTransaction();
            
            ProxyDebug.println
            ("\n***********************************************\nResponse " +
            response.getStatusCode() + " "+response.getReasonPhrase()
            +" received.\n");
            ProxyDebug.println("Processing Response in progress");
            
           
            if (clientTransaction == null)  {
                if (ProxyDebug.debug)  {
                    ProxyDebug.println("The client transaction does not exist, we"+
                    " will forward the response statelessly!");
                }
                
            }
            else {
                ProxyDebug.println("The client transaction exists, here is the "+
                " client transaction: "+clientTransaction);
                
                if (ProxyDebug.debug)  {
                    if (clientTransaction.getDialog()!=null) {
                        ProxyDebug.println("The dialog (isServer:"+clientTransaction.getDialog().isServer()+
                        ") for the client transaction exists,"+
                        " here is the identifier: "+clientTransaction.getDialog().getDialogId());
                        if (clientTransaction.getDialog().getState()!=null) {
                             ProxyDebug.println("The dialog state:"+
                             clientTransaction.getDialog().getState().toString() );
                        }
                        else {
                              ProxyDebug.println("The dialog state is null");
                        }
                    }
                    else
                        ProxyDebug.println("The dialog for the client transaction does not exist");
                }
            }
            
            String method = ((CSeqHeader) response.getHeader(CSeqHeader.NAME)).
            getMethod();
            ProxyDebug.println("DEBUG, Proxy, processResponse(), "+
            "the response to process: method:"+method);
            MethodProcessing methodProcessing= configuration.
		getMethodProcessing(method);
            if (methodProcessing==null) {
                  ProxyDebug.println("DEBUG, Proxy, processResponse(), "+
            "Unable to find the right method: we use DEFAULT_PROCESSING");
                     methodProcessing= configuration.
		getMethodProcessing("DEFAULT_PROCESSING");
                     methodProcessing.processResponse(this,response,clientTransaction);
            }
            else
                methodProcessing.processResponse(this,response,clientTransaction);
        } catch (RuntimeException ex) {
            ex.printStackTrace();
            System.exit(0);
        }
    }
    
    
    /** JAIN Listener method.
     */
    public void processTimeout(TimeoutEvent timeOutEvent) {
        ProxyDebug.println("TimeoutEvent received");
        if (timeOutEvent.isServerTransaction()) {
            ServerTransaction serverTransaction  =
            timeOutEvent.getServerTransaction();
            transactionsMapping.removeMapping(serverTransaction);
        } else {
            ClientTransaction clientTransaction =
            timeOutEvent.getClientTransaction();
            ServerTransaction st = transactionsMapping.
            getServerTransaction(clientTransaction);
            if (st==null) {
                ProxyDebug.println("ERROR, Unable to retrieve the server transaction,"+
                " we cannot process the timeout!");
                return;
            }
            Request request = st.getRequest();
            // This removes the given mapping from the table but not
            // necessarily the whole thing.
            transactionsMapping.removeMapping(clientTransaction);
            if (!transactionsMapping.hasMapping(st)) {
                // No more mappings left in the transaction table.
                try {
                    Response response = messageFactory.createResponse
                    (Response.REQUEST_TIMEOUT, request);
                    sipProvider.sendResponse(st,response);
                } catch (ParseException ex) {
                    ex.printStackTrace();
                } catch (SipException ex1) {
                    ex1.printStackTrace();
                }
            }
        }
        
        
    }
    
    
    
    
    /***********************  Methods for         ***************
     *    starting and stopping the proxy          		*
     ************************************************************/
    
    /** Start the proxy, this method has to be called after the init method
     * throws Exception that which can be caught by the upper application
     */
    public void start() throws Exception {
        if (configuration!=null
        && configuration.isValidConfiguration()) {
            Properties properties=new Properties();
            // LOGGING property:
            
            if (configuration.enableDebug) {
                ProxyDebug.debug=true;
                ProxyDebug.setProxyOutputFile(configuration.outputProxy);
                ProxyDebug.println("DEBUG properties set!");
                if (configuration.badMessageLogFile!=null)
                    properties.setProperty("gov.nist.javax.sip.BAD_MESSAGE_LOG",
                    configuration.badMessageLogFile);
                if (configuration.debugLogFile!=null) {
                    properties.setProperty("gov.nist.javax.sip.DEBUG_LOG",
                    configuration.debugLogFile);
                   
                }
                if (configuration.serverLogFile!=null)
                    properties.setProperty("gov.nist.javax.sip.SERVER_LOG",
                    configuration.serverLogFile);
                if (configuration.debugLogFile != null)
                    properties.setProperty("gov.nist.javax.sip.TRACE_LEVEL",
                    "32");
                else
                if (configuration.serverLogFile != null)
                    properties.setProperty("gov.nist.javax.sip.TRACE_LEVEL",
                    "16");
                else
                    properties.setProperty("gov.nist.javax.sip.TRACE_LEVEL",
                    "0");
           
                
            }
            else {
                System.out.println("DEBUG properties not set!");
            }
             registrar.setExpiresTime(configuration.expiresTime);
            
            // STOP TIME
            if (configuration.stopTime!=null) {
                try {
                    long stopTime=Long.parseLong(configuration.stopTime);
                    StopProxy stopProxy=new StopProxy(this);
                    Timer timer=new Timer();
                    timer.schedule(stopProxy,stopTime);
                }
                catch(Exception e) {
                    e.printStackTrace();
                }
            }
            
            sipStack = null;
            sipProvider = null;
            
            
            SipFactory sipFactory = SipFactory.getInstance();
            sipFactory.setPathName("gov.nist");
            
            headerFactory = sipFactory.createHeaderFactory();
            addressFactory = sipFactory.createAddressFactory();
            messageFactory = sipFactory.createMessageFactory();
            
            // Create SipStack object
            
            properties.setProperty("javax.sip.IP_ADDRESS",
            configuration.stackIPAddress);
            properties.setProperty("javax.sip.STACK_NAME",
            configuration.stackName);
            if (configuration.check(configuration.outboundProxy))
                properties.setProperty("javax.sip.OUTBOUND_PROXY",
                configuration.outboundProxy);
            if (configuration.check(configuration.routerPath))
                properties.setProperty("javax.sip.ROUTER_PATH",
                configuration.routerPath);
            if (configuration.check(configuration.extensionMethods))
                properties.setProperty("javax.sip.EXTENSION_METHODS",
                configuration.extensionMethods);
            if (configuration.check(configuration.retransmissionFilter))
                properties.setProperty("javax.sip.RETRANSMISSION_FILTER",
                configuration.retransmissionFilter);
            
            if (configuration.check(configuration.maxConnections) ) 
                properties.setProperty("gov.nist.javax.sip.MAX_CONNECTIONS",
                configuration.maxConnections);
            if (configuration.check(configuration.maxServerTransactions) ) 
                properties.setProperty("gov.nist.javax.sip.MAX_SERVER_TRANSACTIONS",
                configuration.maxServerTransactions);
            if (configuration.check(configuration.threadPoolSize) ) 
                properties.setProperty("gov.nist.javax.sip.THREAD_POOL_SIZE",
                configuration.threadPoolSize);
            if (configuration.check(configuration.stackDomain) ) 
                stackDomain=configuration.stackDomain;
            
            if (configuration.accessLogViaRMI) {
                properties.setProperty("gov.nist.javax.sip.ACCESS_LOG_VIA_RMI",
                "true");
                
                properties.setProperty("gov.nist.javax.sip.RMI_PORT",
                configuration.logRMIPort);
                
                if (configuration.check(configuration.logLifetime) )
                    properties.setProperty("gov.nist.javax.sip.LOG_LIFETIME",
                    configuration.logLifetime);
                
            }
            
            sipStack = sipFactory.createSipStack(properties);
            
            
            
            // Authentication part:
            if (configuration.enableAuthentication) {
                authentication =new Authentication(this);
                try{
                    
                    Class authMethodClass =Class.forName(configuration.classFile);
                    AuthenticationMethod authMethod
                    = (AuthenticationMethod)
                    authMethodClass.newInstance();
                    authMethod.initialize(configuration.passwordsFile);
                   
                    authentication.setAuthenticationMethod(authMethod);
                    
                }
                catch(Exception e) {
                    ProxyDebug.println
                    ("ERROR, authentication process stopped, exception raised:");
                    e.printStackTrace();
                }
            }

            // We create the Listening points:
            Vector lps=configuration.getListeningPoints();
            
            for ( int i=0;lps!=null && i<lps.size();i++) {
                Association a=(Association)lps.elementAt(i);
                try{
                    System.out.println("transport  " + a.transport);
                    System.out.println("port  " +
                    Integer.valueOf(a.port).intValue());
                    ListeningPoint lp=sipStack.createListeningPoint
                    (Integer.valueOf(a.port).intValue(),
                    a.transport);
                    sipProvider = sipStack.createSipProvider(lp);
                    sipProvider.addSipListener( this );
                }
                catch(Exception e) {
                    e.printStackTrace();
                    ProxyDebug.println
                    ("ERROR: listening point not created ");
                }
            }
            // Export the registry for polling by the responder.
            
            if (configuration.exportRegistry )
                // initialize the registrar for  RMI.
                this.registrar.initRMIBindings();
            
            
            // Parse static configuration for registrar.
            if (configuration.enableRegistrations) {
                String value=configuration.registrationsFile;
                ProxyDebug.println("Parsing the XML registrations file: "+value);
                if (value==null || value.trim().equals(""))
                    ProxyDebug.println("You have to set the registrations file...");
                else
                    registrar.parseXMLregistrations(value);
            }
            else ProxyDebug.println("No registrations to parse...");
            
        }
        else {
            System.out.println("ERROR: the configuration file is not correct!"+
            " Correct the errors first.");
        }
    }
    
    /** Stop the proxy, this method has to be called after the start method
     * throws Exception that which can be caught by the upper application
     */
    public void stop()  throws Exception {
        if (sipStack==null) return;
        Iterator listeningPoints=sipStack.getListeningPoints();
        if (listeningPoints!=null) {
            while( listeningPoints.hasNext()) {
                ListeningPoint lp=(ListeningPoint)listeningPoints.next();
                sipStack.deleteListeningPoint(lp);
                ProxyDebug.println("One listening point removed!");
            }
        }
        else {
            ProxyDebug.println
            ("WARNING, STOP_PROXY, The proxy has no listening "+
            " points to remove!");
        }
        registrar.clean();
    }
    
    /** Exit the proxy,
     * throws Exception that which can be caught by the upper application
     */
    public void exit()  throws Exception {
        Iterator listeningPoints=sipStack.getListeningPoints();
        if (listeningPoints!=null) {
            while( listeningPoints.hasNext()) {
                ListeningPoint lp=(ListeningPoint)listeningPoints.next();
                sipStack.deleteListeningPoint(lp);
                ProxyDebug.println("One listening point removed!");
            }
        }
        else {
            ProxyDebug.println("WARNING, STOP_PROXY, The proxy " +
            " has no listening points to remove!");
        }
        ProxyDebug.println("Proxy exit.........................");
	configuration.listeningPoints.clear();
        registrar.clean();
    }
    
    public ViaHeader getStackViaHeader() {
	try {
	  ListeningPoint lp = 
		(ListeningPoint)sipStack.getListeningPoints().next();
	  String host = sipStack.getIPAddress();
	  int port = lp.getPort();
	  String transport = lp.getTransport();
	  // branch id is assigned by the transaction layer.
	  return  headerFactory.createViaHeader
			(host,port,transport,null);
	} catch (Exception e) {
		e.printStackTrace();
		return null;
	}
	
    }
    
    public ContactHeader getStackContactHeader() {
	try {
	  ListeningPoint lp = 
		(ListeningPoint)sipStack.getListeningPoints().next();
	  String host = sipStack.getIPAddress();
	  int port = lp.getPort();
	  String transport = lp.getTransport();
          
	  SipURI sipURI=addressFactory.createSipURI(null,host);
          sipURI.setPort(port);
          sipURI.setTransportParam(transport);
          Address contactAddress=addressFactory.createAddress(sipURI);
          
          return headerFactory.createContactHeader(contactAddress);
	} catch (Exception e) {
		e.printStackTrace();
		return null;
	}
	
    }
    
    
    /*************************************************************************/
    /************ The main method: to launch the proxy          *************/
    /************************************************************************/
    
    
    public static void main(String args[]) {
        try{
            // the Proxy:
            String confFile= (String) args[1];
            Proxy proxy=new Proxy(confFile);
            proxy.start();
            ProxyDebug.println("Proxy ready to work");
        }
        catch(Exception e) {
            System.out.println
            ("ERROR: Set the configuration file flag: " +
            "USE: -cf configuration_file_location.xml"  );
            System.out.println("ERROR, the proxy can not be started, " +
            " exception raised:\n");
            e.printStackTrace();
        }
    }
    
}

