/******************************************************************************
 * Product of NIST/ITL Advanced Networking Technologies Division (ANTD).      *
 ******************************************************************************/

package gov.nist.javax.sip;
import java.util.*;
import gov.nist.javax.sip.header.*;
import gov.nist.javax.sip.stack.*;
import gov.nist.javax.sip.header.*;
import gov.nist.javax.sip.message.*;
import gov.nist.javax.sip.stack.*;
import javax.sip.message.*;
import javax.sip.header.*;
import javax.sip.address.*;
import javax.sip.*;
import gov.nist.core.*;
import java.io.*;
import gov.nist.javax.sip.address.*;
import java.text.ParseException;


/** Implementation of the JAIN-SIP provider interface.
 *
 *@version  JAIN-SIP-1.1
 *
 *@author M. Ranganathan <mranga@nist.gov>  <br/>
 *
 *<a href="{@docRoot}/uncopyright.html">This code is in the public domain.</a>
 *
 */
public final class SipProviderImpl implements
javax.sip.SipProvider, SIPTransactionEventListener {
    
    protected   SipListener sipListener;
    
    private     LinkedList pendingEvents;
    
    protected boolean isActive;
    
    private     SIPTransactionStack sipStack;
    
    protected   SipStack sipStackImpl;
    
    protected  ListeningPointImpl listeningPoint;
    
    
    /** Stop processing messages for this provider. Post an empty message to our
     *message processing queue that signals us to quit.
     */
    protected synchronized  void stop() {
        // Put an empty event in the queue and post ourselves a message.
        if(LogWriter.needsLogging)
            LogWriter.logMessage("Exiting provider" );
        listeningPoint.removeSipProvider();
    }
    
    
    /**
     * Handle the SIP event - because we have only one listener and we are
     * already in the context of a separate thread, we dont need to enque
     * the event and signal another thread.
     *
     *@param sipEvent is the event to process.
     *
     */
    
    public synchronized void handleEvent(EventObject sipEvent) {
        if (LogWriter.needsLogging) {
            LogWriter.logMessage("handleEvent " + sipEvent );
        }
        if (this.sipListener == null) return;
        if (sipEvent instanceof RequestEvent) {
            sipListener.processRequest((RequestEvent) sipEvent);
        } else if (sipEvent instanceof ResponseEvent) {
            sipListener.processResponse((ResponseEvent) sipEvent);
        } else if (sipEvent instanceof TimeoutEvent) {
            sipListener.processTimeout((TimeoutEvent) sipEvent);
        } else {
            if (LogWriter.needsLogging) {
                LogWriter.logMessage("bad event" + sipEvent);
            }
        }
        
    }
    
    
    
    
    
    
    /** Creates a new instance of SipProviderImpl */
    public SipProviderImpl() {
        this.pendingEvents = new LinkedList();
    }
    
    protected Object clone() throws java.lang.CloneNotSupportedException {
        throw new java.lang.CloneNotSupportedException();
    }
    
    public boolean equals(Object obj) {
        return super.equals(obj);
    }
    
    
    
    
    
    /** This method registers the SipListener object to this SipProvider, once
     * registered the SIP Listener can send events on the SipProvider and
     * recieve events emitted from the SipProvider. As JAIN SIP resticts a
     * unicast Listener special case, that is, that one and only one Listener
     * may be registered on the SipProvider concurrently.
     * <p>
     * If an attempt is made to re-register the existing SipListener this
     * method returns silently. A previous SipListener must be removed from the
     * SipProvider before another SipListener can be registered to
     * the SipProvider.
     *
     * @param <var>SipListener</var> SipListener to be registered with the
     * Provider.
     * @throws TooManyListenersException this exception is thrown when a new
     * SipListener attempts to register with the SipProvider when another
     * SipListener is already registered with this SipProvider.
     *
     */
    public void addSipListener(SipListener sipListener)
    throws TooManyListenersException {
	
	synchronized(sipStackImpl) {
	  Iterator it = sipStackImpl.getSipProviders();
	  while (it.hasNext()) {
	     SipProviderImpl provider = (SipProviderImpl) it.next();
	     if (provider.sipListener  != null && 
		provider.sipListener != sipListener)
                throw new TooManyListenersException();
	   }
	}
        if (LogWriter.needsLogging)
            LogWriter.logMessage("add SipListener " + sipListener);
        this.sipListener = sipListener;
	synchronized(sipStackImpl) {
	  Iterator it = sipStackImpl.getSipProviders();
	  while (it.hasNext()) {
	     SipProviderImpl provider = (SipProviderImpl) it.next();
	     provider.sipListener = sipListener;
	   }
	}
    }
    
    /** Returns the ListeningPoint of this SipProvider.
     * A SipProvider has a single Listening Point at any specific point in time.
     *
     * @see ListeningPoint
     * @return the ListeningPoint of this SipProvider
     */
    public ListeningPoint getListeningPoint() {
        return this.listeningPoint;
    }
    
    /** Returns a unique CallIdHeader for identifying dialogues between two
     * SIP applications.
     *
     * @return new CallId unique within the SIP Stack.
     */
    public CallIdHeader getNewCallId() {
        String callId = Utils.generateCallIdentifier
        (this.getSipStack().getIPAddress());
        CallID callid = new  CallID();
        try {
            callid.setCallId(callId);
        } catch( java.text.ParseException ex) {}
        return callid;
        
    }
    
    /** Once an application wants to a send a new request it must first request
     * a new client transaction identifier. This method is called by an
     * application to create the client transaction befores it sends the Request
     * via the SipProvider on that transaction. This methods returns a new
     * unique client transaction identifier that can be passed to the stateful
     * sendRequest method on the SipProvider and the sendAck/sendBye
     * methods on the Dialog in order to send a request.
     *
     * @param the new Request message that is to handled statefully by the
     * Provider.
     * @return a new unique client transation identifier
     * @see ClientTransaction
     * @since v1.1
     */
    public ClientTransaction getNewClientTransaction(Request request) 
	throws TransactionUnavailableException {
        if (request == null)
            throw new NullPointerException("null request");
        
        if (request.getMethod().equalsIgnoreCase(Request.CANCEL)) {
            SIPClientTransaction ct = (SIPClientTransaction) 
		sipStack.findCancelTransaction((SIPRequest)request,false);
            if (ct != null ) {
                ClientTransaction retval =   sipStack.createClientTransaction
                (ct.getMessageChannel());
		((SIPTransaction)retval).
			setOriginalRequest((SIPRequest)request);
                ((SIPTransaction)retval).addEventListener(this);
		sipStack.addTransaction((SIPClientTransaction)retval);
		((SIPClientTransaction)retval).setDialog
			( (DialogImpl)ct.getDialog());
		return  retval;
            }
            
        }
	if (LogWriter.needsLogging)
		LogWriter.logMessage("could not find existing transaction for "
		+ ((SIPRequest)request).getFirstLine());
        
        Iterator it = sipStackImpl.getRouter().getNextHops(request);
        // could not route the request.
        if (it == null || !it.hasNext() ) return null;
        while (it.hasNext()) {
            Hop hop = (Hop) it.next();
            try {
                SIPClientTransaction ct =
                (SIPClientTransaction) sipStack.createMessageChannel(hop);
                String branchId = Utils.generateBranchId();
                SIPRequest sipRequest = (SIPRequest) request;
		if (sipRequest.getTopmostVia() != null) {
                    sipRequest.getTopmostVia().setBranch(branchId);
		} else {
		    // Find a message processor to assign this transaction to.
		     MessageProcessor messageProcessor = 
			sipStack.getMessageProcessor(hop.getTransport());
		     Via via = messageProcessor.getViaHeader();
		     sipRequest.addHeader(via);
		}
                ct.setOriginalRequest(sipRequest);
		ct.setBranch(branchId);
                if (sipStack.isDialogCreated(request.getMethod())) {
                    // create a new dialog to contain this transaction
                    // provided this is necessary.
		    String dialogId = sipRequest.getDialogId(false);
		    DialogImpl dialog = sipStack.getDialog(dialogId);
		    // This could be a re-invite (but noticed by Brad Templeton)
		    if (dialog != null) 
			ct.setDialog(dialog);
		    else sipStack.createDialog(ct);
                }  else {
		   String dialogId = sipRequest.getDialogId(false);
		   DialogImpl dialog = sipStack.getDialog(dialogId);
		   ct.setDialog(dialog);
		}
                // The provider is the event listener for all transactions.
                ct.addEventListener(this);
                return (ClientTransaction) ct;
            } catch (java.net.UnknownHostException ex) {
                continue;
            } catch (java.text.ParseException ex) {
                InternalErrorHandler.handleException(ex);
            }
        }
        throw new TransactionUnavailableException
		("Could not create transaction - could not resolve next hop! ");
        
    }
    
    /** An application has the responsibility of deciding to respond to a
     * Request that does not match an existing server transaction. The method
     * is called by an application that decides to respond to an unmatched
     * Request statefully. This methods return a new unique server transaction
     * identifier that can be passed to the stateful sendResponse methods in
     * order to respond to the request.
     *
     * @param the initial Request message that the doesn't match an existing
     * transaction that the application decides to handle statefully.
     * @return a new unique server transation identifier
     * @throws TransactionAlreadyExistsException if a transaction already exists
     * that is already handling this Request. This may happen if the application
     * gets retransmits of the same request before the initial transaction is
     * allocated.
     * @see ServerTransaction
     * @since v1.1
     */
    public ServerTransaction
    getNewServerTransaction(Request request)
    throws TransactionAlreadyExistsException,
	TransactionUnavailableException {
        if (sipStack.findTransaction((SIPRequest)request,true) != null)
            throw new TransactionAlreadyExistsException
            ("server transaction already exists!");
        
        Iterator it = sipStack.getRouter().getNextHops(request);
        if (it == null) {
	    throw new TransactionUnavailableException("Null next hop!");
        } else {
	    while (it.hasNext()) {
              try {
                  MessageChannel messageChannel =
                  sipStack.createRawMessageChannel
                   ((Hop)it.next());
                  SIPServerTransaction retval =
                  sipStack.createServerTransaction(messageChannel);
                  retval.addEventListener(this);
                  return (ServerTransaction) retval;
               } catch (java.net.UnknownHostException ex) {
			continue;
               }
	     }
        }
        throw new TransactionUnavailableException
			("Could not route to next hop!");
	
        
    }
    
    /** Returns the SipStack that this SipProvider is attached to. A SipProvider
     * can only be attached to a single SipStack object which belongs to
     * the same SIP stack as the SipProvider.
     *
     * @see SipStack
     * @return the attached SipStack.
     */
    public SipStack getSipStack() {
        return (SipStack) this.sipStackImpl;
    }
    
    /** Removes the SipListener from this SipProvider. This method returns
     * silently if the <var>sipListener</var> argument is not registered
     * with the SipProvider.
     *
     * @param SipListener - the SipListener to be removed from this
     * SipProvider
     */
    public void removeSipListener(SipListener sipListener) {
        if (sipListener == this.sipListener) {
            this.sipListener = null;
        }
    }
    
    
    /** Sends specified {@link javax.sip.message.Request} on a unique
     * client transaction identifier. This method implies that the application
     * is functioning as either a User Agent Client or a Stateful proxy, hence
     * the underlying SipProvider acts statefully.
     * <p>
     * JAIN SIP defines a retransmission utility specific to user agent
     * behaviour and the default retransmission behaviour for each method.
     * <p>
     * When an application wishes to send a message, it creates a Request
     * message passes that Request to this method, this method returns the
     * cleintTransactionId generated by the SipProvider. The Request message
     * gets sent via the ListeningPoint that this SipProvider is attached to.
     * <ul>
     * <li>User Agent Client - must not send a BYE on a confirmed INVITE until
     * it has received an ACK for its 2xx response or until the server
     * transaction times out.
     * </ul>
     *
     * @see Request
     * @see ClientTransaction
     * @param clientTransaction - the new ClientTransaction object identifying
     * this transaction.
     * @param request - the new Request message to send.
     * @throws SipException if implementation cannot send request for any reason
     */
    public void sendRequest(ClientTransaction clientTransaction,
    Request request)
    throws SipException {
        SIPRequest sipRequest = (SIPRequest) request;
        SIPClientTransaction ct = (SIPClientTransaction) clientTransaction;
        try {
            ct.sendMessage(sipRequest);
        } catch (IOException ex) {
            throw new SipException(ex.getMessage());
        }
        
    }
    
    /** Sends specified Response message to a Request which is identified by the
     * specified server transaction identifier. The semantics for various
     * application behaviour on sending Responses to Requests is outlined at
     * {@link SipListener#processRequest(RequestEvent)}.
     * <p>
     * Note that when a UAS core sends a 2xx response to an INVITE, the server
     * transaction is destroyed, by the underlying JAIN SIP implementation.
     * This means that when the ACK sent by the corresponding UAC arrives
     * at the UAS, there will be no matching server transaction for the ACK,
     * and based on this rule, the ACK is passed to the UAS application core,
     * where it is processed.
     * This ensures that the three way handsake of an INVITE that is managed by
     * the UAS application and not JAIN SIP.
     *
     * @param serverTransactionId - server transaction id of the Request upon
     * the Response is sent
     * @param response - the Response to send to the Request
     * @throws TransactionDoesNotExistException if the serverTransactionId does
     * not correspond to any existing server transactions
     * @throws SipException if implementation cannot send response for any
     * other reason
     * @see Response
     * @see ServerTransaction
     */
    public void sendResponse(ServerTransaction serverTransaction,
    Response response)
    throws TransactionDoesNotExistException, SipException {
        if (serverTransaction == null)
            throw new NullPointerException("null serverTransaction");
        DialogImpl dialog  = (DialogImpl) serverTransaction.getDialog();
        // Fix up the response if the dialog has already been established.
        try {
            SIPResponse responseImpl = (SIPResponse) response;
	
	    if ( responseImpl.getStatusCode() == 200 			    && 
	         sipStack.isDialogCreated(responseImpl.getCSeq().getMethod()) &&
		 dialog.getLocalTag() == null 				    && 
		 responseImpl.getTo().getTag()  == null )
		throw new SipException("To tag must be set for OK");

	    if ( responseImpl.getStatusCode() == 200 			    && 
		 responseImpl.getCSeq().getMethod().equals(Request.INVITE)  &&
		 responseImpl.getHeader(ContactHeader.NAME)  == null )
		throw new SipException
			("Contact Header is mandatory for the OK");
            
            // If sending the response within an established dialog, then
            // set up the tags appropriately.
            
            if (  dialog != null && dialog.getLocalTag() != null)
                responseImpl.getTo().setTag(dialog.getLocalTag());
            
	    String fromTag = ((SIPRequest)serverTransaction.getRequest()).
				getFrom().getTag();
            responseImpl.getFrom().setTag(fromTag);
            
            ((SIPServerTransaction)serverTransaction).sendMessage
            ((SIPResponse)response);
            // See if the dialog needs to be inserted into the dialog table
            // or if the state of the dialog needs to be changed.
            if (dialog != null) {
                dialog.printTags();
                if (responseImpl.getCSeq().getMethod().
                equalsIgnoreCase(Request.BYE)) {
                    dialog.setState(DialogImpl.TERMINATED_STATE);
		} else if (responseImpl.getCSeq().getMethod().
			equalsIgnoreCase(Request.CANCEL)) {
		    if (dialog.getState() == null || 
			dialog.getState().getValue()== DialogImpl.EARLY_STATE) {
		        dialog.setState(DialogImpl.TERMINATED_STATE);
		    }
                } else if ( dialog.getLocalTag() == null &&
                     responseImpl.getTo().getTag() != null ) {
                    if (responseImpl.getStatusCode() != 100)
                        dialog.setLocalTag(responseImpl.getTo().getTag());
		    if (sipStack.isDialogCreated(responseImpl.getCSeq().
				getMethod())) {
                        if (response.getStatusCode()/100 == 1) {
                           dialog.setState(DialogImpl.EARLY_STATE);
                        }
                        // Enter into our dialog table provided this is a
		        // dialog creating method.
                        if (responseImpl.getStatusCode() != 100)
                    	    sipStack.putDialog(dialog);
		    }
                } else if (response.getStatusCode()/100 == 2 ) {
		    if (sipStack.isDialogCreated(responseImpl.getCSeq().
				getMethod())) {
                    	dialog.setState(DialogImpl.CONFIRMED_STATE);
                    	sipStack.putDialog(dialog);
		    }
                }
            }
        } catch (IOException ex) {
            throw new SipException(ex.getMessage());
        } catch (java.text.ParseException ex1) {
            throw new SipException(ex1.getMessage());
        }
    }
    
    /** Sends specified {@link javax.sip.message.Request} and returns void i.e.
     * no transaction record is associated with this action. This method
     * implies that the application is functioning statelessly specific to this
     * Request, hence the underlying SipProvider acts statelessly.
     * <p>
     * Once the Request message has been passed to this method, the SipProvider
     * will forget about this Request. No transaction semantics will be
     * associated with the Request and no retranmissions will occur on the
     * Request by the SipProvider, if these semantics are required it is the
     * responsibility of the application not the JAIN SIP Stack.
     * <ul>
     * <li>Stateless Proxy - A stateless proxy simply forwards every request
     *  it receives downstream and discards information about the request
     *  message once the message has been forwarded. A stateless proxy does not
     *  have any notion of a transaction.
     * </ul>
     *
     * @since v1.1
     * @see Request
     * @param request - the Request message to send statelessly
     * @throws SipException if implementation cannot send request for any reason
     */
    public void sendStatelessRequest(Request request) throws SipException {
        Iterator it = sipStack.getRouter().getNextHops(request);
        if (it == null || !it.hasNext())
            throw new SipException("could not determine next hop!");
        while (it.hasNext()) {
            Hop nextHop = (Hop) it.next();
            try {
		SIPRequest sipRequest = (SIPRequest) request;
		String bid = sipRequest.getTransactionId();
		Via via = sipRequest.getTopmostVia();
		via.setBranch(bid);
		// Do not create a transaction for this request.
                MessageChannel messageChannel =
                sipStack.createRawMessageChannel(nextHop);
		if (messageChannel != null) 
                 	messageChannel.sendMessage((SIPMessage)request);
		else throw new SipException ("could not forward request");
            } catch (IOException ex) {
                throw new SipException(ex.getMessage());
            } catch (ParseException ex1) {
		InternalErrorHandler.handleException(ex1);
	    }
        }
        
    }
    
    /** Sends specified {@link javax.sip.message.Response} and returns void i.e.
     * no transaction record is associated with this action. This method implies
     * that the application is functioning as either a stateless proxy or a
     * stateless User Agent Server.
     * <ul>
     *  <li> Stateless proxy - A stateless proxy simply forwards every response
     *  it receives upstream and discards information about the response message
     *  once the message has been forwarded. A stateless proxy does not
     *  have any notion of a transaction.
     *  <li>Stateless User Agent Server - A stateless UAS does not maintain
     *  transaction state. It replies to requests normally, but discards
     *  any state that would ordinarily be retained by a UAS after a response
     *  has been sent.  If a stateless UAS receives a retransmission of a
     *  request, it regenerates the response and resends it, just as if it
     *  were replying to the first instance of the request. A UAS cannot be
     *  stateless unless the request processing for that method would always
     *  result in the same response if the requests are identical. Stateless
     *  UASs do not use a transaction layer; they receive requests directly
     *  from the transport layer and send responses directly to the transport
     *  layer.
     * </ul>
     *
     * @see Response
     * @param response - the Response to send statelessly.
     * @throws SipException if implementation cannot send
     *   response for any reason
     * @see Response
     * @since v1.1
     */
    public void sendStatelessResponse(Response response)
    throws SipException {
        SIPResponse sipResponse = (SIPResponse) response;
        Via via = sipResponse.getTopmostVia();
        String host = via.getHost();
        int port = via.getPort();
        String transport = via.getTransport();
        Hop hop = new HopImpl(host+":"+port+"/" +transport);
        try {
            MessageChannel messageChannel = 
			sipStack.createRawMessageChannel(hop);
            messageChannel.sendMessage(sipResponse);
        } catch (IOException ex) {
            throw new SipException(ex.getMessage());
        }
    }
    
    /** This method sets the listening point of the SipProvider.
     * A SipProvider can only have a single listening point at any
     * specific time. This method returns
     * silently if the same <var>listeningPoint</var> argument is re-set
     * on the SipProvider.
     * <p>
     * JAIN SIP supports recieving messages from
     * any port and interface that a server listens on for UDP, on that same
     * port and interface for TCP in case a message may need to be sent
     * using TCP, rather than UDP, if it is too large. In order to satisfy this
     * functionality an application must create two SipProviders and set
     * identical listeningPoints except for transport on each SipProvder.
     * <p>
     * Multiple SipProviders are prohibited to listen on the same
     * listening point.
     *
     * @param the <var>listeningPoint</var> of this SipProvider
     * @see ListeningPoint
     * @since v1.1
     */
    public void setListeningPoint(ListeningPoint listeningPoint)  {
        if (listeningPoint == null)
            throw new NullPointerException("Null listening point");
        ListeningPointImpl lp = (ListeningPointImpl) listeningPoint;
        lp.sipProviderImpl = this;
        this.listeningPoint = (ListeningPointImpl) listeningPoint;
        
    }
    
    /** This is part of the implementation.
     */
    protected void setSipStack(SipStackImpl sipStackImpl) {
        this.sipStackImpl = sipStackImpl;
        this.sipStack = (SIPTransactionStack) sipStackImpl;
    }
    
    /** 	Invoked when an error has ocurred with a transaction.
     *Propagate up to the listeners.
     *
     * 	@param transactionErrorEvent Error event.
     */
    public void transactionErrorEvent
    (SIPTransactionErrorEvent transactionErrorEvent) {
        SIPTransaction transaction =
        (SIPTransaction) transactionErrorEvent.getSource();
        
        if (transactionErrorEvent.getErrorID() ==
        SIPTransactionErrorEvent.TRANSPORT_ERROR) {
            if (LogWriter.needsLogging) {
                LogWriter.logMessage
                ("TransportError occured on " + transaction);
            }
            
        } else {
            // This is a timeout event.
            Object errorObject = transactionErrorEvent.getSource();
            Timeout timeout = Timeout.TRANSACTION;
            TimeoutEvent ev = null;
            
            if (errorObject instanceof SIPServerTransaction) {
                ev = new TimeoutEvent(this,(ServerTransaction)
                errorObject,timeout);
            } else {
                ev = new TimeoutEvent(this,(ClientTransaction) errorObject,
                timeout);
            }
            this.handleEvent(ev);
            
        }
    }
    
}
