/*******************************************************************************
* Product of NIST/ITL Advanced Networking Technologies Division (ANTD).        *
*******************************************************************************/
package gov.nist.javax.sip.header;
import javax.sip.InvalidArgumentException;
import javax.sip.header.*;
import java.text.ParseException;
import gov.nist.core.*;
/**
* Accept-Encoding SIP (HTTP) Header.
*
*@author M. Ranganathan <mranga@nist.gov> 
*@author Olivier Deruelle <deruelle@nist.gov><br/>
*<a href="{@docRoot}/uncopyright.html">This code is in the public domain.</a>
*
* <pre>
* From HTTP RFC 2616
*
* 
*   The Accept-Encoding request-header field is similar to Accept, but
*   restricts the content-codings (section 3.5) that are acceptable in
*   the response.
*
* 
*       Accept-Encoding  = "Accept-Encoding" ":"
* 
* 
*                          1#( codings [ ";" "q" "=" qvalue ] )
*       codings          = ( content-coding | "*" )
* 
*   Examples of its use are:
* 
*       Accept-Encoding: compress, gzip
*       Accept-Encoding:
*       Accept-Encoding: *
*       Accept-Encoding: compress;q=0.5, gzip;q=1.0
*       Accept-Encoding: gzip;q=1.0, identity; q=0.5, *;q=0
* </pre>
* 
*/
public class AcceptEncoding extends ParametersHeader
implements AcceptEncodingHeader {
    
        /** qvalue field
         */        
        protected Float qvalue;
	
        /** contentEncoding field
         */        
        protected String contentCoding;
	
        /** default constructor
         */        
        public AcceptEncoding() {
		super(NAME);
	}
	
        /**
         * Encode the header in canonical form.
         * @return encoded header.
         */
	public String encode() {
		String encoding =  headerName + COLON;
		if (contentCoding != null) {
		   encoding  += SP + contentCoding;
		 }
		if (qvalue != null ) {
			encoding += SEMICOLON + "q" + EQUALS + qvalue;
		}
                if (!parameters.isEmpty()) {
                    encoding+=SEMICOLON+parameters.encode();
                }
		encoding += NEWLINE;
		return encoding;
	}        

	/** Encode the value of this header.
	*@return the value of this header encoded into a string.
	*/
	protected String encodeBody() {
		String encoding = "";
		if (contentCoding != null) {
		   encoding  +=  contentCoding;
		 }
		if (qvalue != null ) {
			encoding += SEMICOLON + "q" + EQUALS + qvalue;
		}
		return encoding;
	}        
        
        /** get QValue field
         * @return float
         */        
        public float getQValue() { 
	    if (qvalue == null) return -1; // means not set.
            else return qvalue.floatValue();
        }
	
        /** get ContentEncoding field
         * @return String
         */        
        public String getEncoding() { 
            return contentCoding;
        }
        
	/**
         * Set the qvalue member
         * @param q double to set
         */
	public	 void setQValue(float q) throws InvalidArgumentException{ 
		if (q < 0.0 || q > 1.0) 
			throw new InvalidArgumentException
				("qvalue out of range!");
            qvalue = new Float(q) ;
        }
        
	/**
     * Sets the encoding of an EncodingHeader.
     *
     * @param encoding - the new string value defining the encoding.
     * @throws ParseException which signals that an error has been reached
     * unexpectedly while parsing the encoding value.
     */

    public void setEncoding(String encoding) throws ParseException {
         if (encoding==null) 
	throw new  NullPointerException("JAIN-SIP Exception, AcceptEncoding, "+
       "setEncoding(), the "+
            " encoding parameter is null"); 
         contentCoding=encoding;
    }
        
}
