/*******************************************************************************
 * Product of NIST/ITL Advanced Networking Technologies Division (ANTD).       *
 ******************************************************************************/
package gov.nist.javax.sip.header;
import gov.nist.core.*;
import gov.nist.javax.sip.*;
import javax.sip.header.*; 
import  java.util.Hashtable;
import  java.util.Enumeration;
import  java.util.ListIterator;
import gov.nist.javax.sip.address.*;

/**
*  Keeps a list and a hashtable of via header functions.
*
*@version  JAIN-SIP-1.1
*
*@author M. Ranganathan <mranga@nist.gov>  <br/>
*
*<a href="{@docRoot}/uncopyright.html">This code is in the public domain.</a>
*
*/

public final class ViaList extends SIPHeaderList {
    
    private  Hashtable viaTable; // For quick address lookups.
    
    private String stringRep;
    
         /**
          * Constructor.
          * @param hl SIPObjectList to set
          */
    public ViaList(SIPObjectList hl) {
        super(hl,VIA);
        viaTable = new Hashtable();
    }
    
         /**
          * Default Constructor.
          */
    public ViaList() {
        super( SIPHEADERS_PACKAGE+".Via", ViaHeader.NAME);
        viaTable = new Hashtable();
    }
    
        /**
         * Add to our linked list and also put it in our table.
         * @param viaHdr SIPObject to set.
         */
    public void add ( SIPObject viaHdr ) {
        super.add(viaHdr);
        Via via = (Via) viaHdr;
        viaTable.put(via.sentBy.encode(),via);
    }
    
        /**
         * Concatenation function - concatenates a given via list at either end
         * of this list.
         * The hash tables for quick access to via lists are merged and the
         * lists themselves are merged by using the concatenation method of the
         * superclass. This function supercedes the parent function of the same
         * signature.
         * @param otherVia other list to concatenate with this list.
         * @param topFlag if set to true other list is prepended to us.
         * @exception IllegalArgumentException if the given header list is not a
         * list of via headers.
         */
    public void concatenate( SIPHeaderList otherVia, boolean topFlag )
    throws IllegalArgumentException {
        ViaList other = (ViaList) otherVia;
        Enumeration keys = other.viaTable.keys();
        super.concatenate(otherVia,topFlag);
        viaTable.putAll(other.viaTable);
    }
    
        /**
         * make a clone of this header list. This supercedes the parent
         * function of the same signature.
         * @return clone of this Header.
         */
    public Object clone() {
        ViaList vlist = new ViaList();
	ListIterator it = this.listIterator();
	while (it.hasNext()) {
		Via v = (Via) ((Via) it.next()).clone();
                vlist.add(v);
        }
        return (Object) vlist;
    }
    
        /**
         * Concatenation function - append the given list to  this list.
         * @param otherVia other list to concatenate with this list.
         * @exception IllegalArgumentException if the given header list is not a
         * list of via headers.
         */
    public void concatenate( SIPHeaderList otherVia)
    throws IllegalArgumentException  {
        this.concatenate(otherVia,false);
    }
    
        /**
         *  Is this address on the sent-by list?
         * A proxy server can use this function
         *  to aid in loop detection.
         * @param address String to set.
         * @return true if the Header contains the address.
         */
    public boolean isAddressOnViaList( String address ) {
        return viaTable.containsKey(address.trim().toLowerCase()) ;
    }
    
        /**
         * function for debug
         * @param str String to set.
         */
    public void sprint( String str) {
        stringRep += str  ;
        stringRep += "\n";
    }
    
        /**
         *  string formatting function.
         * @param indentation int to set.
         * @return string formatting.
         */
    public String debugDump(int indentation) {
        Enumeration keys = viaTable.keys();
        stringRep = "";
        String indent  =
        new Indentation(indentation).getIndentation();;
        this.sprint( indent + "Keys : ");
        while (keys.hasMoreElements()) {
            String key = (String) keys.nextElement();
            sprint(indent + key);
        }
        this.sprint(indent + "--------------------------");
        sprint(this.hlist.debugDump(indentation));
        return stringRep;
    }
    
        /**
         *  function for debug.
         *  @return String.
         */
    public String debugDump() {
        return this.debugDump(0);
    }

	/** Return an encoding for this header.
	*
	* @return encoded version of the header.
	*/
	public String toString() { 
		return this.encode(); 
	}

	/**
	* Replace an object from this contact list.
	*@param objectText Encoded form of the object that we want to 
	*  replace.
	*@param replacementObject object that we want to put in the place
	*   of this object.
	*@param matchSubstring flag that indicates whether we want to pick
	* targets on substring matches.
	*/
	public void replace(String objectText, 
			GenericObject replacementObject,
			boolean matchSubstring ) {
		
	    if (replacementObject instanceof Via) {
		Via via = (Via) replacementObject;
		HostPort hp = via.sentBy;
		if (hp != null) {
                	String sb = hp.encode();
			viaTable.remove(sb);
		}
	    } 
	    super.replace(objectText,replacementObject,matchSubstring);
	}
    
}
