/**************************************************************************/
/* Product of NIST Advanced Networking Technologies Division		  */
/**************************************************************************/
package gov.nist.javax.sip.stack;

import java.util.*;
import gov.nist.javax.sip.header.*;
import gov.nist.javax.sip.parser.*;
import gov.nist.javax.sip.address.*;
import gov.nist.javax.sip.message.*;
import gov.nist.core.*;
import javax.sip.header.*;
import javax.sip.*;
import javax.sip.address.*;
import javax.sip.message.*;
import java.io.IOException;
import java.net.*;
import java.text.ParseException;


/** Tracks dialogs. A dialog is a peer to peer association of communicating
 * SIP entities. For INVITE transactions, a Dialog is created when a success
 * message is received (i.e. a response that has a To tag).
 * The SIP Protocol stores enough state in the
 * message structure to extract a dialog identifier that can be used to
 * retrieve this structure from the SipStack.
 *
 *@version  JAIN-SIP-1.1
 *
 *@author M. Ranganathan <mranga@nist.gov>  <br/>
 *
 *<a href="{@docRoot}/uncopyright.html">This code is in the public domain.</a>
 *
 *
 */

public class DialogImpl implements javax.sip.Dialog {
    private SIPTransaction firstTransaction;
    private SIPTransaction lastTransaction;
    private String dialogId;
    private int localSequenceNumber;
    private int remoteSequenceNumber;
    private String myTag;
    private String hisTag;
    private Hashtable routeSet;
    private RouteList  routeList;
    private Route  	   contactRoute;
    private String user;
    private Route defaultRoute;
    private SIPTransactionStack sipStack;
    private int dialogState;
    protected boolean ackSeen;
    private GenericURI requestURI;
    private boolean rrFlag;
    // protected String allocationKey;
    
    public final static int EARLY_STATE     = DialogState._EARLY;
    public final static int CONFIRMED_STATE = DialogState._CONFIRMED;
    public final static int COMPLETED_STATE = DialogState._COMPLETED;
    public final static int TERMINATED_STATE = DialogState._TERMINATED;
    
    
    
    /**
     *A debugging print routine.
     */
    private void printRouteList() {
        if (LogWriter.needsLogging) {
            LogWriter.logMessage("printRouteList : " +
            this.routeList.encode());
            if (this.contactRoute != null) {
                LogWriter.logMessage("contactRoute : " +
                this.contactRoute.encode());
            } else {
                LogWriter.logMessage("contactRoute : null");
            }
        }
    }
    
    
    
    /** Return true if this is a client dialog.
     *
     *@return true if the transaction that created this dialog is a
     *client transaction and false otherwise.
     */
    public boolean isClientDialog() {
        SIPTransaction transaction =
        (SIPTransaction) this.getFirstTransaction();
        return transaction instanceof SIPClientTransaction;
    }
    
    /**
     *Return true if record routing is enabled for this dialog
     */
    public boolean recordRouteEnabled() {
        return rrFlag;
    }
    
    /** Set the state for this dialog.
     *
     *@param state is the state to set for the dialog.
     */
    
    public void setState(int state) {
	if (LogWriter.needsLogging) {
	    LogWriter.logMessage("Setting dialog state for " +
			this);
	    LogWriter.logStackTrace();
	    if (state != -1 && 
		state != this.dialogState ) 
		if (LogWriter.needsLogging) {
		   LogWriter.logMessage("New dialog state is " + 
			DialogState.getObject(state) +  
			"dialogId = " +
			this.getDialogId() );
		}

	}
        this.dialogState = state;
    }
    
    /** Debugging print for the dialog.
     */
    public void printTags() {
        if (LogWriter.needsLogging) {
            LogWriter.logMessage( "isServer = " + isServer());
            LogWriter.logMessage("localTag = " + getLocalTag());
            LogWriter.logMessage( "remoteTag = " + getRemoteTag());
            LogWriter.logMessage("firstTransaction = " +
            ((SIPTransaction) firstTransaction).getOriginalRequest());
            
        }
    }
    
    
    /** Mark that the dialog has seen an ACK.
     */
    public void ackReceived(SIPRequest sipRequest) {
	if (isServer()) {
	    SIPServerTransaction st = 
		 (SIPServerTransaction)this.getFirstTransaction();
	    if (st == null) return;
	    // Suppress retransmission of the final response (in case
	    // retransmission filter is being used).
	    if (st.getOriginalRequest().getCSeq().getSequenceNumber() ==
		sipRequest.getCSeq().getSequenceNumber()) {
			st.setState(SIPTransaction.TERMINATED_STATE);
	    }
	}
    }
    
    
    /** Get the transaction that created this dialog.
     * This is the first dialog that is not cancelled on the list
     * of transactions managed by this dialog. This depends on the
     * tag assignment.
     */
    public Transaction getFirstTransaction() {
        if (this.getRemoteTag() == null) return null;
	return this.firstTransaction;
    }
    
    /** Gets the route set for the dialog.
     * When acting as an User Agent Server
     * the route set MUST be set to the list of URIs in the
     * Record-Route header field from the request, taken in order and
     * preserving all URI parameters. When acting as an User Agent
     * Client the route set MUST be set to the list of URIs in the
     * Record-Route header field from the response, taken in
     * reverse order and preserving all URI parameters. If no Record-Route
     * header field is present in the request or response,
     * the route set MUST be set to the empty set. This route set,
     * even if empty, overrides any
     * pre-existing route set for future requests in this dialog.
     * <p>
     * Requests within a dialog MAY contain Record-Route
     * and Contact header fields.
     * However, these requests do not cause the dialog's route set to be
     * modified.
     * <p>
     * The User Agent Client uses the remote target
     * and route set to build the
     * Request-URI and Route header field of the request.
     *
     * @return an Iterator containing a list of route headers to be used for
     *  forwarding. Empty iterator is returned if route has not 
     * 	been established.
     */
     public Iterator getRouteSet() {
	  RouteList rl =  this.getRouteList() ;
	   if (routeList != null) return rl.listIterator();
	   else return new RouteList().listIterator();
     }
    
/**
    public Iterator getRouteSet() {
        if (this.getUser() != null && this.defaultRoute != null )
            ((AddressImpl) this.defaultRoute.getAddress()).setUser
            (this.getUser());
        // Find the top via in the route list.
        ListIterator li = routeList.listIterator(routeList.size());
        boolean flag = false;
	// Initialize the route list.
	// Bug reported by Brad Templeton
        RouteList retval = new RouteList();
        
        while (li.hasPrevious()) {
                // Find the via header in the route list.
                Route route = (Route) li.previous();
		String host = ((SipUri)route.getAddress().getURI()).getHost();
		int port = ((SipUri)route.getAddress().getURI()).getPort();
		if (port == -1) port = 5060;
		String transport = 
			((SipUri)route.getAddress().getURI()).
			getTransportParam();
		if (transport == null) transport = "udp";
                if (flag) {
                    if (retval == null)  {
                        this.requestURI =
                        (SipUri) route.getAddress().getURI();
                    }
                    retval.add(route);
                }
		if (host.equalsIgnoreCase(sipStack.getHostAddress()) &&
			sipStack.isTransportEnabled(transport,port) ) {
		    flag = true;
		}
       }
        
        return retval.listIterator();
    }
**/
    
    
    /** A conveniance function that appends the contact header to
     * the end of the route set and returns the cloned value.
     */
    private RouteList getRouteList() {
        if (this.getUser() != null && this.defaultRoute != null )
            ((SipUri)this.defaultRoute.getAddress()).setUser
            (this.getUser());
        // Find the top via in the route list.
        ListIterator li = routeList.listIterator(routeList.size());
        boolean flag = true;
        RouteList retval = new RouteList();
        while (li.hasPrevious()) {
           // Find the via header in the route list.
           Route route = (Route) li.previous();
	   String host = ((SipUri)route.getAddress().getURI()).getHost();
	   int port = ((SipUri)route.getAddress().getURI()).getPort();
	    if (port == -1) port = 5060;
	    String transport = 
			((SipUri)route.getAddress().getURI()).
			getTransportParam();
	    if (flag && host.equalsIgnoreCase(sipStack.getHostAddress()) &&
			sipStack.isTransportEnabled(transport,port) ) {
		    flag = false;
	    } else retval.add(route.clone());
        }

        if ( contactRoute != null) {
	     Route route = (Route) contactRoute.clone();
	    // knock off the user name.
	     ((SipUri)route.getAddress().getURI()).removeUser();
             retval.add(route);
         }

        if (LogWriter.needsLogging) {
            LogWriter.logMessage("----->>> " );
            if (retval != null)
                LogWriter.logMessage("RouteList = " +
                retval.encode());
            LogWriter.logMessage("myRouteList = " +
            routeList.encode());
            LogWriter.logMessage("----->>> " );
        }
        return retval;
    }
    
    
    /** Set the stack address.
     * Prevent us from routing messages to ourselves.
     *
     *@param stackAddress the address of the SIP stack.
     *
     */
    public void setStack(SIPTransactionStack sipStack) {
        this.sipStack = sipStack;
    }
    
    
    /**
     * Add a route to the dialog.
     *
     *@param route is the route to add to the dialog.
     *
     */
    public void addRoute(Route route, boolean top) {
        // In the case of a user agent, this should be initialized to
        // the proxy address
        routeList.add(route);
    }
    
    
    /**
     * Generate a request URI from the top of the route list.
     *
     *@return a request uri generated from the top of the route list.
     */
    public GenericURI getOriginalRequestURI() {
        if (this.requestURI != null) return this.requestURI;
        if (routeList.size() == 0) return null;
        RouteList routeList = this.getRouteList();
        if (routeList == null) return null;
        Route route = (Route) routeList.getFirst();
        SipUri retval =   (SipUri) route.getAddress().getURI();
	retval.setUser(this.getUser());
	return retval;
	
    }
    
    
    /**
     * Set the default route (the default next hop for the proxy or
     * the proxy address for the user agent).
     *
     *@param defaultRoute is the default route to set.
     *
     */
    
    public void setDefaultRoute(Route defaultRoute) {
        this.defaultRoute = (Route) defaultRoute.clone();
        // addRoute(defaultRoute,false);
    }
    
    /**
     * Set the user name for the default route.
     *
     *@param user is the user name to set for the default route.
     *
     */
    public void setUser(String user) {
        this.user = user;
    }
    
    
    /** Add a route list extracted from a record route list.
     * If this is a server dialog then we assume that the record
     * are added to the route list IN order. If this is a client
     * dialog then we assume that the record route headers give us
     * the route list to add in reverse order.
     *
     *@param recordRouteList -- the record route list from the incoming
     *      message.
     */
    
    private void addRoute(RecordRouteList recordRouteList) {
        if (this.isClientDialog()) {
            // This is a client dialog so we extract the record
            // route from the response and reverse its order to
            // careate a route list.
            this.routeList =  new RouteList();
            // start at the end of the list and walk backwards
            ListIterator li = recordRouteList.listIterator
            (recordRouteList.size());
            while (li.hasPrevious()) {
                RecordRoute rr = (RecordRoute) li.previous();
                Route route = new Route();
                AddressImpl addr = (AddressImpl) rr.getAddress();
                route.setAddress
                ((AddressImpl)((AddressImpl)rr.getAddress()).clone());
                route.setParameters((NameValueList)rr.getParameters().clone());
                this.routeList.add(route);
            }
        } else {
            // This is a server dialog. The top most record route
            // header is the one that is closest to us. We extract the
            // route list in the same order as the addresses in the
            // incoming request.
            this.routeList = new RouteList();
            ListIterator li = recordRouteList.listIterator();
            while(li.hasNext()) {
                RecordRoute rr = (RecordRoute) li.next();
                Route route = new Route();
                route.setAddress
			((AddressImpl)((AddressImpl) rr.getAddress()).clone());
                route.setParameters((NameValueList)rr.getParameters().
                clone());
                routeList.add(route);
            }
        }
    }
    
    
    /** Add a route list extacted from the contact list of the incoming
     *message.
     *
     *@param contactList -- contact list extracted from the incoming
     *  message.
     *
     */
    
    private void addRoute(ContactList contactList) {
        if (contactList.size() == 0 ) return;
        Contact contact = (Contact) contactList.getFirst();
        Route route = new Route();
        route.setAddress
        ((AddressImpl)((AddressImpl)(contact.getAddress())).clone());
        this.contactRoute = route;
    }
    
    
    /**
     * Extract the route information from this SIP Message and
     * add the relevant information to the route set.
     *
     *@param sipMessage is the SIP message for which we want
     *  to add the route.
     *
     *@param SIPMessage is the incoming SIP message from which we
     * want to extract out the route.
     */
    public synchronized void addRoute(SIPMessage sipMessage) {
        // cannot add route list after the dialog is initialized.
	if (LogWriter.needsLogging) {
		LogWriter.logMessage("addRoute: dialogState: "+this.getState());
	}
        if ( this.dialogState == CONFIRMED_STATE ||
	this.dialogState == COMPLETED_STATE ||
        this.dialogState == TERMINATED_STATE) return;
        if (!isServer()) {
            // I am CLIENT dialog.
            if (sipMessage instanceof SIPResponse) {
                RecordRouteList rrlist = sipMessage.getRecordRouteHeaders();
                // Add the route set from the incoming response in reverse
                // order
                if (rrlist != null ) {
                    this.rrFlag = true;
                    this.addRoute(rrlist);
                } else {
                    // Set the rotue list to the last seen route list.
                    this.rrFlag = false;
                    this.routeList = new RouteList();
                }
                ContactList contactList = sipMessage.getContactHeaders();
                if (contactList != null) {
                    this.addRoute(contactList);
                }
            }
        }  else  {
            if (sipMessage instanceof SIPRequest) {
                // Incoming Request has the route list
                RecordRouteList rrlist = sipMessage.getRecordRouteHeaders();
                // Add the route set from the incoming response in reverse
                // order
                if (rrlist != null ) {
                    this.rrFlag = true;
                    
                    this.addRoute(rrlist);
                } else {
                    // Set the rotue list to the last seen route list.
                    this.rrFlag = false;
                    this.routeList = new RouteList();
                }
                // put the contact header from the incoming request into
                // the route set.
                ContactList contactList = sipMessage.getContactHeaders();
                if (contactList != null) {
                    this.addRoute(contactList);
                }
            }
        }
        if (LogWriter.needsLogging)  {
            LogWriter.logMessage
            ("added a route = " + routeList.encode());
        }
    }
    
    
    
    /** Protected Dialog constructor.
     */
    private DialogImpl() {
        this.routeSet = new Hashtable();
        this.routeList = new RouteList();
        this.dialogState =  -1; // not yet initialized.
        localSequenceNumber = 1;
        remoteSequenceNumber = -1;
    }
    
    
    /** Set the dialog identifier.
     */
    public void setDialogId(String dialogId) {
        this.dialogId = dialogId;
    }
    
    /** Constructor given the first transaction.
     *
     *@param transaction is the first transaction.
     */
    protected DialogImpl(SIPTransaction transaction) {
        this();
        this.addTransaction(transaction);
    }
    
    /** Get the user name for the dialog.
     */
    public String getUser() { return firstTransaction.getUser(); }
    
    
    
    /** Return true if is server.
     *
     *@return true if is server transaction created this dialog.
     */
    public boolean isServer() {
        return this.firstTransaction instanceof SIPServerTransaction;
        
    }
    
    
    
    
    
    /** Get the id for this dialog.
     *
     *@return the string identifier for this dialog.
     *
     */
    public String getDialogId() {
	
        if (firstTransaction instanceof SIPServerTransaction ) {
            if (true || this.dialogId == null) {
                SIPRequest sipRequest = (SIPRequest)
                ((SIPServerTransaction) firstTransaction).getRequest();
                this.dialogId = sipRequest.getDialogId(true,this.myTag);
            }
            
        } else {
            // This is a server transaction. Compute the dialog id
            // from the tag we have assigned to the outgoing
            // response of the dialog creating transaction.
    		if (   this.firstTransaction != null && 
			((SIPClientTransaction)this.firstTransaction).
			getLastResponse() != null) {
                   this.dialogId =
                   ((SIPClientTransaction)firstTransaction).getLastResponse().
                   getDialogId(false,this.hisTag);
	    }
            
        }

        return this.dialogId;
    }
    
    /**
     * Add a transaction record to the dialog.
     *
     *@param transaction is the transaction to add to the dialog.
     */
    public  void addTransaction(SIPTransaction transaction) {
        // Dont add transactions to terminated dialogs.
	
        if (firstTransaction == null  ) {
            // Record the local and remote sequenc
            // numbers and the from and to tags for future
            // use on this dialog.
            firstTransaction = transaction;
            SIPRequest sipRequest =
            (SIPRequest) transaction.getOriginalRequest();
            
            if (transaction instanceof SIPServerTransaction ) {
                setRemoteSequenceNumber
                (sipRequest.getCSeq().getSequenceNumber());
                hisTag = sipRequest.getFrom().getTag();
                // My tag is assigned when sending response
            } else {
                setLocalSequenceNumber
                (sipRequest.getCSeq().getSequenceNumber());
                // his tag is known when receiving response
                myTag = sipRequest.getFrom().getTag();
		if (myTag == null) 
			throw new RuntimeException("bad message tag missing!");
            }
        }
	this.lastTransaction = transaction;
        // set a back ptr in the incoming dialog.
        transaction.setDialog(this);
        if (LogWriter.needsLogging) {
            LogWriter.logMessage("Transaction Added " +
            this + myTag + "/" + hisTag);
            LogWriter.logMessage("TID = "
            + transaction.getTransactionId() +
            "/" + transaction.IsServerTransaction() );
            LogWriter.logStackTrace();
        }
    }
    
    /**
     * Set the remote tag.
     *
     *@param hisTag is the remote tag to set.
     */
    public void setRemoteTag(String hisTag) {
        this.hisTag = hisTag;
    }
    
    
    
    
    /**
     * Get the last transaction from the dialog.
     */
    public SIPTransaction getLastTransaction() {
        return  this.lastTransaction;
        
    }
    
    
    
    /**
     * Set the local sequece number for the dialog (defaults to 1 when
     * the dialog is created).
     *
     *@param lCseq is the local cseq number.
     *
     */
    protected void setLocalSequenceNumber(int lCseq) {
        this.localSequenceNumber = lCseq;
    }
    
    /**
     * Set the remote sequence number for the dialog.
     *
     * @param rCseq is the remote cseq number.
     *
     */
    protected void setRemoteSequenceNumber(int  rCseq) {
        this.remoteSequenceNumber = rCseq;
    }
    
    /**
     * Increment the Remote cseq # for the dialog.
     *
     * @return the incremented remote sequence number.
     *
     */
    public int  incrementRemoteSequenceNumber() {
        return ++this.remoteSequenceNumber;
    }
    
    /**
     * Increment the local CSeq # for the dialog.
     *
     *@return the incremented local sequence number.
     *
     */
    public int incrementLocalSequenceNumber() {
        return ++this.localSequenceNumber;
    }
    
    /**
     * Get the remote sequence number (for cseq assignment of outgoing
     * requests within this dialog).
     *
     *@return local sequence number.
     */
    public int getRemoteSequenceNumber() {
        return this.remoteSequenceNumber;
    }
    
    /**
     * Get the local sequence number (for cseq assignment of outgoing
     * requests within this dialog).
     *
     *@return local sequence number.
     */
    
    public int getLocalSequenceNumber() {
        return this.localSequenceNumber;
    }
    
    
    /**
     * Get local identifier for the dialog.
     *  This is used in From header tag construction
     *  for all outgoing client transaction requests for
     *  this dialog and for all outgoing responses for this dialog.
     *  This is used in To tag constuction for all outgoing
     *  transactions when we are the server of the dialog.
     *  Use this when constucting To header tags for BYE requests
     *  when we are the server of the dialog.
     *
     *@return the local tag.
     */
    public String getLocalTag() { 
	 return this.myTag;
    }
    
    /** Get peer identifier identifier for the dialog.
     * This is used in To header tag construction for all outgoing
     * requests when we are the client of the dialog.
     * This is used in From tag construction for all outgoing
     * requests when we are the Server of the dialog. Use
     * this when costructing From header Tags for BYE requests
     * when we are the server of the dialog.
     *
     *@return the remote tag
     *  (note this is read from a response to an INVITE).
     *
     */
    public String getRemoteTag() {
        return hisTag;
    }
    
    /** Set local tag for the transaction.
     *
     *@param mytag is the tag to use in From headers client
     * 	transactions that belong to this dialog and for
     *	generating To tags for Server transaction requests that belong
     * 	to this dialog.
     */
    public void setLocalTag(String mytag) {
        if (LogWriter.needsLogging)  {
            LogWriter.logMessage("set Local tag " + mytag + " "
            + this.dialogId);
            LogWriter.logStackTrace();
        }
        
        this.myTag = mytag;
	
    }
    
    
    
    /** Mark all the transactions in the dialog inactive and ready
     * for garbage collection.
     */
    protected  void deleteTransactions() {
	this.firstTransaction = null;
	this.lastTransaction = null;
    }
    
    
    
    /** This method will release all resources associated with this dialog
     * that are tracked by the Provider. Further references to the dialog by
     * incoming messages will result in a mismatch.
     * Since dialog destruction is left reasonably open ended in RFC3261,
     * this delete method is provided
     * for future use and extension methods that do not require a BYE to
     * terminate a dialogue. The basic case of the INVITE and all dialogues
     * that we are aware of today it is expected that BYE requests will
     * end the dialogue.
     */
    
    public void delete() {
        // the reaper will get him later.
        this.setState(TERMINATED_STATE);
    }
    
    /** Returns the Call-ID for this SipSession. This is the value of the
     * Call-ID header for all messages belonging to this session.
     *
     * @return the Call-ID for this Dialogue
     */
    public CallIdHeader getCallId() {
        SIPRequest sipRequest =
        ((SIPTransaction)this.getFirstTransaction()).getOriginalRequest();
        return sipRequest.getCallId();
    }
    
    
    /** Get the local Address for this dialog.
     *
     *@return the address object of the local party.
     */
    
    public javax.sip.address.Address getLocalParty()  {
        SIPRequest sipRequest =
        ((SIPTransaction)this.getFirstTransaction()).getOriginalRequest();
        return sipRequest.getFrom().getAddress();
    }
    
    /**
     * Returns the Address identifying the remote party.
     * This is the value of the To header of locally initiated
     * requests in this dialogue when acting as an User Agent Client.
     * <p>
     * This is the value of the From header of recieved responses in this
     * dialogue when acting as an User Agent Server.
     *
     * @return the address object of the remote party.
     */
    public javax.sip.address.Address getRemoteParty() {
	if (!isServer()) {
          SIPRequest sipRequest =
          ((SIPTransaction)this.getFirstTransaction()).getOriginalRequest();
          return sipRequest.getTo().getAddress();
	} else  {
          SIPResponse sipResponse =
          ((SIPTransaction)this.getFirstTransaction()).getLastResponse();
          return sipResponse.getFrom().getAddress();
	}
        
    }
    
    /**
     * Returns the Address identifying the remote target.
     * This is the value of the Contact header of recieved Responses
     * for Requests or refresh Requests
     * in this dialogue when acting as an User Agent Client <p>
     * This is the value of the Contact header of recieved Requests
     * or refresh Requests in this dialogue when acting as an User
     * Agent Server.
     *
     * @return the address object of the remote target.
     */
    public javax.sip.address.Address getRemoteTarget() {
        return this.contactRoute.getAddress();
    }
    
    
    
    /** Returns the current state of the dialogue. The states are as follows:
     * <ul>
     * <li> Early - A dialog is in the "early" state, which occurs when it is
     * created when a provisional response is recieved to the INVITE Request.
     * <li> Confirmed - A dialog transitions to the "confirmed" state when a 2xx
     * final response is received to the INVITE Request.
     * <li> Completed - A dialog transitions to the "completed" state when a BYE
     * request is sent or received by the User Agent Client.
     * <li> Terminated - A dialog transitions to the "terminated" state when it
     * can be garbage collection.
     * </ul>
     * Independent of the method, if a request outside of a dialog generates a
     * non-2xx final response, any early dialogs created through provisional
     * responses to that request are terminated. If no response arrives at all
     * on the early dialog, it also terminates.
     *
     * @return a DialogState determining the current state of the dialog.
     * @see DialogState
     */
    public DialogState getState() {
	if (this.dialogState == -1) return null; // not yet initialized
        return DialogState.getObject(this.dialogState);
    }
    
    /** Returns true if this Dialog is secure i.e. if the request arrived over
     * TLS, and the Request-URI contained a SIPS URI, the "secure" flag is set
     * to TRUE.
     *
     * @return  <code>true</code> if this dialogue was established using a sips
     * URI over TLS, and <code>false</code> otherwise.
     */
    public boolean isSecure() {
        return ((SipUri)this.getOriginalRequestURI()).getScheme().
        equalsIgnoreCase("sips");
    }
    
    /** Sends ACK Request to the remote party of this Dialogue. This method
     * implies that the application is functioning as User Agent Client hence
     * the underlying SipProvider acts statefully. This method should only be
     * used for an INVITE transactions, all ACK's to non-INVITE transactions
     * should use the 
     * {@link gov.nist.javax.sip.SipProviderImpl#sendRequest(ClientTransaction, Request)} method.
     * <p>
     * When the {@link gov.nist.javax.sip.SipStackImpl#getRetransmissionFilter()} 
     * is <code>true</code>, that is the SipProvider takes care of 
     * all retransmissions for the application, if the the SipProvider can 
     * not deliver the Request after
     * multiple retransmits due to the transaction expiring the SipListener will
     * be notified with a {@link javax.sip.TimeoutEvent}.
     *
     * @param ackRequest - the new ACK Request message to send.
     * @throws SipException if implementation cannot send the ACK Request for
     * any other reason
     */
    public void sendAck(Request request) throws SipException {
        SIPRequest ackRequest = (SIPRequest) request;
        if (this.isServer())
            throw new SipException("Cannot sendAck from Server side of Dialog");
        if (!ackRequest.getMethod().equals(Request.ACK)) 
            throw new SipException("Bad request method -- should be ACK");
	if (this.getState() == null || 
	    this.getState().getValue() == EARLY_STATE)  {
	    throw new SipException
	    ("Bad dialog state " + this.getState());
	}
	if (! ((SIPTransaction)this.getFirstTransaction()).
	   getOriginalRequest().getCallId().
	   getCallId().equals(((SIPRequest)request).
			getCallId().getCallId())) {
	    throw new SipException ("Bad call ID in request");
	}
	try {
	       if (LogWriter.needsLogging) {
		  LogWriter.logMessage("setting from tag For outgoing ACK= "  
			+ this.getLocalTag());
		  LogWriter.logMessage("setting To tag for outgoing ACK = "
			+ this.getRemoteTag());
	       }
	       if (this.getLocalTag() != null) 
	       ackRequest.getFrom().setTag(this.getLocalTag());
	       if (this.getRemoteTag() != null) 
	         ackRequest.getTo().setTag(this.getRemoteTag());
	} catch (ParseException ex) {
		throw new SipException(ex.getMessage());
	}
	if (ackRequest.getHeader(RouteHeader.NAME) == null) {
	  // get the route list from the dialog.
	  RouteList rl = this.getRouteList();
	  // Add it to the header.
	  if (rl != null) ackRequest.addHeader(rl);
	}
        
        try {
            SIPClientTransaction clientTransaction = (SIPClientTransaction)
            this.getFirstTransaction();
            clientTransaction.sendMessage((SIPMessage)ackRequest);
	    this.localSequenceNumber++;
        } catch( IOException ex) {
            throw new SipException(ex.getMessage());
        }
        
    }
    
    /**
     * Sends a Request to the remote party of this dialog. This method 
     * implies that the application is functioning as UAC hence the 
     * underlying SipProvider acts statefully. This method is useful for 
     * sending Bye's for terminating a dialog or Re-Invites on the Dialog 
     * for third party call control.
     * <p>
     * This methods will set the From and the To tags for the outgoing 
     * request and also set the correct sequence number to the outgoing 
     * Request and associate the client transaction with this dialog. 
     * Note that any tags assigned by the user will be over-written by this 
     * method. 
     * <p>
     * The User Agent must not send a BYE on a confirmed INVITE until it has 
     * received an ACK for its 2xx response or until the server transaction 
     * timeout is received.
     * <p>
     * When the {@link SipStack#getRetransmissionFilter()} is <code>true</code>, 
     * that is the SipProvider takes care of all retransmissions for the 
     * application, and the SipProvider can not deliver the Request after 
     * multiple retransmits the SipListener will be notified with a 
     * {@link TimeoutEvent} when the transaction expires.
     *
     * @param request - the new Request message to send.
     * @param clientTransaction - the new ClientTransaction object identifying 
     * this transaction, this clientTransaction should be requested from 
     * {@link SipProvider#getNewClientTransaction(Request)}
     * @throws TransactionDoesNotExistException if the serverTransaction does
     * not correspond to any existing server transaction.
     * @throws SipException if implementation cannot send the Request for 
     * any reason.
     */
    public void sendRequest(ClientTransaction clientTransactionId,
        Request dialogRequest) throws
	TransactionDoesNotExistException,SipException {
	
        SIPRequest sipRequest = (SIPRequest) dialogRequest;
	if (clientTransactionId == null || sipRequest == null) 
		throw new NullPointerException("null parameter");

	// Cannot send bye until the dialog has been established.
	if ( this.getState() == null ) {
	    throw new SipException
	    ("Bad dialog state " + this.getState());

	}
	if (sipRequest.getMethod().equalsIgnoreCase(Request.BYE) &&
	    this.getState().getValue() == EARLY_STATE)  {
	    throw new SipException
	    ("Bad dialog state " + this.getState());
	}

	if (sipRequest.getTopmostVia() == null) {
		Via via = 
		((SIPClientTransaction) clientTransactionId).
		getOutgoingViaHeader();
		sipRequest.addHeader(via);
	}


	if (! ((SIPTransaction)this.getFirstTransaction()).
	   getOriginalRequest().getCallId().
	   getCallId().equals(((SIPRequest)sipRequest).
			getCallId().getCallId())) {
	    throw new SipException ("Bad call ID in request");
	}

	// Set the dialog back pointer.
         ((SIPClientTransaction)clientTransactionId).dialog = this;

	if (((SIPClientTransaction)clientTransactionId).getOriginalRequest() 
		!=   sipRequest) {
		throw new SipException("request mismatch! " );
	}


        From from = (From) sipRequest.getFrom();
        To to = (To) sipRequest.getTo();

	try {
	  if (true || ! this.isServer()) {
		from.setTag(this.getLocalTag());
		to.setTag(this.getRemoteTag());
	  } else {
		// Sending the request from the server side. 
		// Tags are inverted.
		from.setTag(this.getRemoteTag());
		to.setTag(this.getLocalTag());
	  }
	} catch (ParseException ex) {
		System.out.println("Huh??");
		ex.printStackTrace();
	}

	// Caller has not assigned the route header - set the route header
	// and the request URI for the outgoing request.

	if (sipRequest.getHeader(RouteHeader.NAME) == null) {
	  // get the route list from the dialog.
	  RouteList rl = this.getRouteList();
	  Route topRoute = (Route) rl.getFirst();
	  // Add it to the header.
	  if (rl != null && !rl.isEmpty()) sipRequest.addHeader(rl);
	  sipRequest.setRequestURI(topRoute.getAddress().getURI());
	  Iterator it = sipStack.getRouter().getNextHops(sipRequest);
	  if (it == null || !it.hasNext()) 
		throw new SipException("Could not route request");
	   Hop hop = (Hop) it.next();
	   try {
	      MessageChannel messageChannel = 
		   sipStack.createRawMessageChannel(hop);
	      ((SIPClientTransaction) clientTransactionId).
			encapsulatedChannel = messageChannel;
	  } catch (Exception ex) {
		if (LogWriter.needsLogging)
		   LogWriter.logException(ex);
		throw new SipException("Cold not create message channel");
	  }
	} else {
		RouteList rl = sipRequest.getRouteHeaders();
	        Route topRoute = (Route) rl.getFirst();
		// Set the request URI.
	        sipRequest.setRequestURI(topRoute.getAddress().getURI());
	}


	
        
        try {
	 ((SIPRequest)sipRequest).getCSeq().
		setSequenceNumber(getLocalSequenceNumber());
	 localSequenceNumber ++;
	} catch (InvalidArgumentException ex) { ex.printStackTrace(); }
        
        if (this.isServer()) {
            SIPServerTransaction serverTransaction = (SIPServerTransaction)
            this.getFirstTransaction();

	    // Use the same encapsulated message channel. This is for TCP
	    // Processing - try to reuse the connection.
	    // if (serverTransaction.isReliable())
	    // ((SIPClientTransaction)clientTransactionId).encapsulatedChannel =
	    //  ((SIPServerTransaction) serverTransaction).encapsulatedChannel ;

            SIPRequest originalRequest  =
            serverTransaction.getOriginalRequest();
            From originalFrom = (From) originalRequest.getFrom();
            To originalTo  = (To) originalRequest.getTo();

	    
	    try {
		  from.setTag(this.myTag);
		  to.setTag(this.hisTag);
	    } catch (ParseException ex) {
		throw new SipException(ex.getMessage());
	    }
            

            try {
                ((SIPClientTransaction) clientTransactionId).sendMessage
			((SIPMessage)sipRequest);
		this.setState(COMPLETED_STATE);
            } catch (IOException ex) {
                throw new SipException("error sending message");
            }
        } else {
            // I am the client so I do not swap headers.
            SIPClientTransaction clientTransaction = (SIPClientTransaction)
            this.getFirstTransaction();
            SIPRequest originalRequest =
            clientTransaction.getOriginalRequest();
            
            From originalFrom = (From) originalRequest.getFrom();
            To originalTo  = (To) originalRequest.getTo();
	    try {

         	if (LogWriter.needsLogging) {
			LogWriter.logMessage("setting tags from " +
				this.getDialogId());      
			LogWriter.logMessage("fromTag " + this.myTag);
			LogWriter.logMessage("toTag " + this.hisTag);
		}
		  
	   	from.setTag(this.myTag);
		to.setTag(this.hisTag);
	    } catch (ParseException ex) {
		throw new SipException(ex.getMessage());
	    }
            
            try {
                ((SIPClientTransaction)clientTransactionId).
			sendMessage((SIPMessage)sipRequest);
		// go directly to terminated state.
		if (sipRequest.getMethod().equalsIgnoreCase(Request.BYE))
		    this.delete();
            } catch (IOException ex) {
                throw new SipException("error sending message");
            }
            
        }
    }
    
}
