/*******************************************************************************
* Product of NIST/ITL Advanced Networking Technologies Division (ANTD).        *
*******************************************************************************/

package gov.nist.javax.sip.stack;
import gov.nist.javax.sip.stack.*;
import gov.nist.javax.sip.*;
import gov.nist.javax.sip.header.*;
import gov.nist.javax.sip.message.*;
import java.util.LinkedList;
import java.util.ListIterator;

/**
* This class is in charge of periodical cleaning.
* Objects which require periodical cleaning register here.
* They must implement the Cleanable interface.
* 
* Design note: this replaces the old Manager classes.
* Each of them was in charge of periodical cleaning for
* a specific class and was running its own thread. Now, there
* is a single thread running, which is in charge of all cleaning.
*
*@version  JAIN-SIP-1.1
*
*@author M. Ranganathan <mranga@nist.gov>  <br/>
*@author Marc Bednarek  <br/>
*<a href="{@docRoot}/uncopyright.html">This code is in the public domain.</a>
*
*/

public class Janitor implements Runnable {
    
    private LinkedList subscribers;
    private int        sleepingTime;
    private Thread     janitor;

    /**
     * Constructor
     * @param sleepingTime The period between each cleaning
     */
 
    public Janitor(Integer sleepingTime) {
	subscribers = new LinkedList();
	janitor = new Thread(this);
	janitor.start();
	if (sleepingTime != null) {
	    this.sleepingTime = sleepingTime.intValue() * 1000;
	} else {
	    this.sleepingTime = 10000; /* 10 sec */
	} 
    }

    /**
     * Add a new object to be periodically cleaned
     * @param object The object to be cleaned (must implement Cleanable)
     */

    public synchronized void register(Cleanable object) {
	subscribers.addLast(object);
    }

    /**
     * Remove from the list an object to be cleaned
     * @param object The object to remove 
     */

    public synchronized  void unregister(Cleanable object) {
	subscribers.remove(object);
    }

    /** 
     * This is the code that periodically clean the objects
     */

    public void run() {
	while (true) {
	    try {
		janitor.sleep(sleepingTime);
	    } catch (InterruptedException ex) {
		// Ignore
	    }
	    synchronized(this) {
	      ListIterator iterator = subscribers.listIterator();
	      while (iterator.hasNext()) {
		Cleanable object = (Cleanable) iterator.next();
		// Joyfully cleans the mess
		object.clean();
	      }
	    }
	}
    }

}

