/*******************************************************************************
* Product of NIST/ITL Advanced Networking Technologies Division (ANTD).        *
*******************************************************************************/

package gov.nist.javax.sip.stack;
import java.rmi.*;
import java.rmi.server.*;
import java.util.*;
import java.net.URLEncoder;

/**
* This class stores all the message logs accessible via RMI
* Here is the format of the XML file:
*<pre>
* <message from=identifier for the machine where the message originated
* to= identifier for the destination
* time= time the message was logged
* statusMessage= un-interpreted status information
* transactionId= Transaction identifier (for coloring the arrows).
* firstLine= first line of the message
* <![CDATA[ Message content ]]>
* ...
*<messages>
*
* For example:
* <message from="127.0.0.1:2000"
* to="127.0.0.1:5070"
* time="1013714883983"
* statusMessage="FSM Nodes Fired by Incoming Msg:  node3"
* transactionId="127.0.0.1:1:INVITE:127.0.0.1:2000"
* firstLine="INVITE sip:127.0.0.1:5070;transport=UDP SIP/2.0">
* <![CDATA[INVITE sip:127.0.0.1:5070;transport=UDP SIP/2.0
* Via: SIP/2.0/UDP 127.0.0.1:2000;branch=fcab1368e30f422f3d9bbdcd5293398c.1
* Via: SIP/2.0/UDP 127.0.0.1:5071
* Record-Route: <sip:127.0.0.1:2000;transport=UDP;maddr=127.0.0.1>
* CSeq: 1 INVITE
* Call-Id: 679a7c8a02d479d0187ca51087d5fe3e@127.0.0.1
* From: <sip:BigGuy@here.com>
*To: <sip:LittleGuy@there.com>
* Content-Length: 0
* ]]>
* ...
* </messages>
*</pre>
*
*@version  JAIN-SIP-1.1
*
*@author M. Ranganathan <mranga@nist.gov>  <br/>
*@author Marc Bednarek 
*
*<a href="{@docRoot}/uncopyright.html">This code is in the public domain.</a>
*
*/

class MessageLogTableImpl extends UnicastRemoteObject 
    implements MessageLogTable, Cleanable {

    // A table that 
    private Hashtable remoteLogTable;
    private HashSet remotePathTable;
    private Hashtable localLogTable;
    private Hashtable auxInfoTable;
    private final int defaultLifeTime = 3600; /* 1 hour */
    private int lifeTime;
    private Hashtable timeOut;
    private String logRootName;
    private String rmiHostName;


    /**
     * Constructor
     */
 
    public MessageLogTableImpl() throws RemoteException {
        super();
	this.auxInfoTable = new Hashtable();
	this.localLogTable = new Hashtable();
	this.remoteLogTable = new Hashtable();
	this.remotePathTable = new HashSet();
	this.timeOut = new Hashtable();
    }

    public MessageLogTableImpl(int port) throws RemoteException {
        super(port);
	this.auxInfoTable = new Hashtable();
	this.localLogTable = new Hashtable();
	this.remoteLogTable = new Hashtable();
	this.remotePathTable = new HashSet();
	this.timeOut = new Hashtable();
    }

    public void init(String rmiHostName, int port, String logRootName)  
		throws RemoteException {
	init(rmiHostName, port, logRootName, defaultLifeTime);
    }

    public void init(String rmiHostName, int port,  
	String logRootName, int lifeTime) 
	throws RemoteException {
	try {
	  //this.logRootName = URLEncoder.encode(logRootName,"UTF-8");
            this.logRootName = logRootName;
	  this.rmiHostName = rmiHostName;
	  if (lifeTime == 0) {
	      this.lifeTime = defaultLifeTime;
	  } else {
	      this.lifeTime = lifeTime;
	  }
       /*
	try { 
	  UnicastRemoteObject.exportObject(this);
	} catch (Exception ex) {}
	*/
           if (System.getSecurityManager() == null) {
	      System.setSecurityManager(new RMISecurityManager());
	   }
	   String name = "//" + rmiHostName + ":" + port + "/" + 
	     	logRootName +"/" + getClass().getName();
	    System.out.println("rebind : "  + name);
	    Naming.rebind(name, this);
	    System.out.println(getClass() + " bound");
	} catch (Exception e) {
	    System.err.println(getClass() + " exception: " + 
			       e.getMessage());
	    e.printStackTrace();
	    throw new RemoteException("problem binding " + e.getMessage());
	}
    }

    public void add(String callId, MessageLogList logList) {
	String logname = "//"+ logRootName + "/"+ callId;
	this.localLogTable.put(logname, logList);
	Calendar date = GregorianCalendar.getInstance();
	date.add(Calendar.SECOND, lifeTime);
	timeOut.put(logname, date);
    }

    /** Get the log for a given logId.
    *@param callId is the call id for which to get the log.
    *@return a MessageLogList from the local table for the given callId.
    */
    public MessageLogList get( String callId) {
	String logId = "//"+ logRootName + "/"+ callId;
	if (this.localLogTable.containsKey(logId)) {
	    return (MessageLogList) this.localLogTable.get(logId);
	} else {
	    return null;
	}
    }
    /** Put a call flow template in to the log table. This is for 
    * use by the trace viewer. The responder can use this to register
    * a template for later viewing by the trace viewer.
    * @param rmiPath is the RMI path for this log.
    */
    public void registerRemoteLog( String rmiPath) 
		throws RemoteException {
	remotePathTable.add(rmiPath);
        // Remove any remote path information
    }


    /** Accept a remote log submission (cache it for access by remote clients.
    * @param name is the key with which to associate the log 
    * (should match the rmi path above).
    * @param log is the submitted log.
    */
    public void submitRemoteLog(String name, String log, String auxInfo) 
	throws RemoteException {
	// System.out.println("Remote log " + name) ;
	// System.out.println("submitted -- "+auxInfo);
	if (log != null) remoteLogTable.put(name,log);
	if (auxInfo != null) auxInfoTable.put(name,auxInfo);
    }


    /** Return the keys for which we have stored information. 
    * This is for the benifit
    * of remote callers. This function pulls in all the 
    * remotely registered logs and returns a set of keys for all the 
    * logs we have to the remote caller.
    * Beware -- there could be circularities in defining remote logs.
    *@return a space separated list of keys to the remote caller.
    */
    public String keys() throws RemoteException {
	Iterator it = remotePathTable.iterator();
	while (it.hasNext()) {
	     String rmiPath  = (String) it.next();
	     try {
	           MessageLogTable table = 
			(MessageLogTable) Naming.lookup(rmiPath);
	           String keys = table.keys();
		   StringTokenizer tokenizer = new StringTokenizer(keys);
		   while (tokenizer.hasMoreElements()) {
			String key = (String) tokenizer.nextElement();
			String value = table.flush(key);
			remoteLogTable.put(key,value);
		   }
	     } catch (Exception ex) {
		continue;
	     }

	}
	Enumeration e = this.localLogTable.keys();
	String result = "";
	while (e.hasMoreElements()) {
	    result += e.nextElement() + " ";
	}
	e = remoteLogTable.keys();
	while (e.hasMoreElements()) {
	    result += e.nextElement() + " ";
	}
	// TODO
	// Query the registered remote logs for keys.
	return result;
    }

    /** Return all our information to a remote caller.
    *@param key is the log id -- could refer to a 
    * locally generated log or a remotely submitted log.
    */
    public synchronized String flush(String key) throws RemoteException {
	if (this.localLogTable.get(key) != null) {
            return (String) ((MessageLogList) 
			this.localLogTable.get(key)).flush();
	} else if (remoteLogTable.containsKey(key)) {
	    // See if a remote log has been sent to us.
	    return (String) remoteLogTable.get(key);
	} else {
	   return null;
	}
    }

    /** Gets any auxiliary information that may be stored here. (For example,
     * in our test system we are storing the XML for the responder).
     *@param logName is the name for which we are storing the remote information.
     */
    public String getAuxInfo(String logName) throws RemoteException {
	
	// System.out.println("!!! for logname : "
	//		+logName+" -- "+auxInfoTable.get(logName)+" !!!") ;
	return (String) auxInfoTable.get(logName);
	
    }

    /** Clean the logs.
    */
    public synchronized void clean() {
	Enumeration keys = timeOut.keys();
	while (keys.hasMoreElements()) {
	    String callId = (String) keys.nextElement();
	    Calendar timeOutDate = (Calendar) timeOut.get(callId);
	    Calendar date = GregorianCalendar.getInstance();
	    if (date.after(timeOutDate)) {
		timeOut.remove(callId);
		this.localLogTable.remove(callId);
	    }
	}
    }

}
