package gov.nist.javax.sip.stack;

import gov.nist.javax.sip.message.*;
import gov.nist.javax.sip.header.*;
import gov.nist.javax.sip.stack.*;
import gov.nist.javax.sip.*;
import gov.nist.core.*;
import gov.nist.javax.sip.SIPConstants;
import javax.sip.message.*;

import javax.sip.*;

import java.io.IOException;


/**
 *Represents a client transaction.
 *
 *@author Jeff Keyser
 *@author M. Ranganathan <mranga@nist.gov>
 *(Modified Jeff's code and aligned with JAIN SIP 1.1)<br/>
 *
 *@version  JAIN-SIP-1.1
 *
 *
 *<a href="{@docRoot}/uncopyright.html">This code is in the public domain.</a>
 *
 *<pre>
 *
 * Implements the following state machines. (From RFC 3261)
 *
 *                               |INVITE from TU
 *             Timer A fires     |INVITE sent
 *             Reset A,          V                      Timer B fires
 *             INVITE sent +-----------+                or Transport Err.
 *               +---------|           |---------------+inform TU
 *               |         |  Calling  |               |
 *               +-------->|           |-------------->|
 *                         +-----------+ 2xx           |
 *                            |  |       2xx to TU     |
 *                            |  |1xx                  |
 *    300-699 +---------------+  |1xx to TU            |
 *   ACK sent |                  |                     |
 *resp. to TU |  1xx             V                     |
 *            |  1xx to TU  -----------+               |
 *            |  +---------|           |               |
 *            |  |         |Proceeding |-------------->|
 *            |  +-------->|           | 2xx           |
 *            |            +-----------+ 2xx to TU     |
 *            |       300-699    |                     |
 *            |       ACK sent,  |                     |
 *            |       resp. to TU|                     |
 *            |                  |                     |      NOTE:
 *            |  300-699         V                     |
 *            |  ACK sent  +-----------+Transport Err. |  transitions
 *            |  +---------|           |Inform TU      |  labeled with
 *            |  |         | Completed |-------------->|  the event
 *            |  +-------->|           |               |  over the action
 *            |            +-----------+               |  to take
 *            |              ^   |                     |
 *            |              |   | Timer D fires       |
 *            +--------------+   | -                   |
 *                               |                     |
 *                               V                     |
 *                         +-----------+               |
 *                         |           |               |
 *                         | Terminated|<--------------+
 *                         |           |
 *                         +-----------+
 *
 *                 Figure 5: INVITE client transaction
 *
 *
 *                                   |Request from TU
 *                                   |send request
 *               Timer E             V
 *               send request  +-----------+
 *                   +---------|           |-------------------+
 *                   |         |  Trying   |  Timer F          |
 *                   +-------->|           |  or Transport Err.|
 *                             +-----------+  inform TU        |
 *                200-699         |  |                         |
 *                resp. to TU     |  |1xx                      |
 *                +---------------+  |resp. to TU              |
 *                |                  |                         |
 *                |   Timer E        V       Timer F           |
 *                |   send req +-----------+ or Transport Err. |
 *                |  +---------|           | inform TU         |
 *                |  |         |Proceeding |------------------>|
 *                |  +-------->|           |-----+             |
 *                |            +-----------+     |1xx          |
 *                |              |      ^        |resp to TU   |
 *                | 200-699      |      +--------+             |
 *                | resp. to TU  |                             |
 *                |              |                             |
 *                |              V                             |
 *                |            +-----------+                   |
 *                |            |           |                   |
 *                |            | Completed |                   |
 *                |            |           |                   |
 *                |            +-----------+                   |
 *                |              ^   |                         |
 *                |              |   | Timer K                 |
 *                +--------------+   | -                       |
 *                                   |                         |
 *                                   V                         |
 *             NOTE:           +-----------+                   |
 *                             |           |                   |
 *         transitions         | Terminated|<------------------+
 *         labeled with        |           |
 *         the event           +-----------+
 *         over the action
 *         to take
 *
 *                 Figure 6: non-INVITE client transaction
 *
 *
 *</pre>
 *
 */
public class SIPClientTransaction
extends SIPTransaction
implements SIPServerResponseInterface, javax.sip.ClientTransaction {
    
    private SIPRequest lastRequest;
    
    
    private int viaPort;
    
    private String viaHost;
    
    // Real ResponseInterface to pass messages to
    private SIPServerResponseInterface	respondTo;
    
    
    /**
     *	Creates a new client transaction.
     *
     *	@param newSIPStack Transaction stack this transaction
     *      belongs to.
     *	@param newChannelToUse Channel to encapsulate.
     */
    protected SIPClientTransaction(
    SIPTransactionStack	newSIPStack,
    MessageChannel		newChannelToUse
    ) {
        super( newSIPStack, newChannelToUse );
        // Create a random branch parameter for this transaction
        // setBranch( SIPConstants.BRANCH_MAGIC_COOKIE +
        // Integer.toHexString( hashCode( ) ) );
        setBranch( Utils.generateBranchId());
        if (LogWriter.needsLogging) {
            LogWriter.logMessage("Creating clientTransaction " + this);
            LogWriter.logStackTrace();
        }
        
    }
    
    
    /**
     *	Sets the real ResponseInterface this transaction encapsulates.
     *
     *	@param newRespondTo ResponseInterface to send messages to.
     */
    public void setResponseInterface(
    SIPServerResponseInterface	newRespondTo
    ) {
        
        respondTo = newRespondTo;
        
    }
    
    
    public String getProcessingInfo(
    ) {
        
        return respondTo.getProcessingInfo( );
        
    }
    
    
    /**
     *	Returns this transaction.
     */
    public MessageChannel getRequestChannel(
    ) {
        
        return this;
        
    }
    
    
    /**
     *	Deterines if the message is a part of this transaction.
     *
     *	@param messageToTest Message to check if it is part of this
     *		transaction.
     *
     *	@return True if the message is part of this transaction,
     * 		false if not.
     */
    public  boolean isMessagePartOfTransaction(
    SIPMessage	messageToTest
    ) {
        
        // List of Via headers in the message to test
        ViaList	viaHeaders = messageToTest.getViaHeaders( );
        // Flags whether the select message is part of this transaction
        boolean transactionMatches;
        String messageBranch = ((Via)viaHeaders.getFirst()).getBranch();
        boolean rfc3261Compliant =
        getBranch() != null &&
        messageBranch != null &&
        getBranch().startsWith(SIPConstants.BRANCH_MAGIC_COOKIE) &&
        messageBranch.startsWith(SIPConstants.BRANCH_MAGIC_COOKIE);

	/**
	if (LogWriter.needsLogging)  {
		LogWriter.logMessage("--------- TEST ------------");
		LogWriter.logMessage(" testing " + this.getOriginalRequest());
		LogWriter.logMessage("Against " + messageToTest);
		LogWriter.logMessage("isTerminated = " + isTerminated());
		LogWriter.logMessage("messageBranch = " + messageBranch);
		LogWriter.logMessage("viaList = " + messageToTest.getViaHeaders());
		LogWriter.logMessage("myBranch = " + getBranch());
	}
	**/
        
        transactionMatches = false;
        if( !isTerminated( ) ) {
            if (rfc3261Compliant) {
                if( viaHeaders != null ) {
                    // If the branch parameter is the
                    //same as this transaction and the method is the same,
                    if( getBranch().equals
                    (((Via)viaHeaders.getFirst()).
                    getBranch())) {
                        transactionMatches =
                        getOriginalRequest().getCSeq().
                        getMethod().equals
                        ( messageToTest.getCSeq().
                        getMethod());
                        
                    }
                }
            } else {
                transactionMatches =
                getOriginalRequest().getTransactionId().equals
                (messageToTest.getTransactionId());
                
            }
            
        }
        return transactionMatches;
        
    }
    
    
    /**
     *  Send a request message through this transaction and
     *  onto the client.
     *
     *	@param messageToSend Request to process and send.
     */
    public void sendMessage(
    SIPMessage	messageToSend
    ) throws IOException {
        
        // Message typecast as a request
        SIPRequest	transactionRequest;
        
        
        transactionRequest = (SIPRequest)messageToSend;
        
        
        // Set the branch id for the top via header.
        Via topVia =
        (Via) transactionRequest.getViaHeaders().getFirst();
        // Tack on a branch identifier  to match responses.
        try {
            topVia.setBranch(getBranch());
        } catch (java.text.ParseException ex) {}
        
        // If this is the first request for this transaction,
        if( getState( ) == null ) {
            // Save this request as the one this transaction
            // is handling
            setOriginalRequest( transactionRequest );
            
            // Change to trying/calling state
            if (transactionRequest.getMethod().equals(Request.INVITE)) {
                setState(CALLING_STATE);
            } else {
                setState( TRYING_STATE );
            }
            if( !isReliable( ) ) {
                
                enableRetransmissionTimer( );
                
            }
            
            if (isInviteTransaction()) {
                enableTimeoutTimer( TIMER_B );
            }  else {
                enableTimeoutTimer( TIMER_F );
            }
            
        } else if  (getState().getValue() == PROCEEDING_STATE ||
        getState().getValue() == CALLING_STATE ) {
            
            // If this is a TU-generated ACK request,
            if( transactionRequest.getMethod().equals( Request.ACK ) ) {
                // Send directly to the underlying
                // transport and close this transaction
                setState( TERMINATED_STATE );
                getMessageChannel().sendMessage
                (transactionRequest );
                return;
                
            }
            
        }
        try {
            
            // Send the message to the server
            lastRequest = transactionRequest;
            getMessageChannel( ).sendMessage( transactionRequest );
            
        } catch( IOException e ) {
            
            setState( TERMINATED_STATE );
            throw e;
            
        }
        
    }
    
    
    /**
     *	Process a new response message through this transaction.
     * If necessary, this message will also be passed onto the TU.
     *
     *	@param transactionResponse Response to process.
     *	@param sourceChannel Channel that received this message.
     */
    public  void processResponse(
    SIPResponse		transactionResponse,
    MessageChannel	sourceChannel
    ) throws SIPServerException {
        // Log the incoming response in our log file.
        if (ServerLog.needsLogging(ServerLog.TRACE_MESSAGES))
            this.logResponse(transactionResponse,
            System.currentTimeMillis(),"normal processing");
        this.lastResponse = transactionResponse;
	if (dialog != null) {
	    // add the route before you process the response.
	    // Bug noticed by Brad Templeton.
            dialog.addRoute(transactionResponse);
	}
        String method = transactionResponse.getCSeq().getMethod();
        if (dialog != null) {
            boolean added = false;
	    SIPTransactionStack sipStackImpl
                = (SIPTransactionStack) getSIPStack();
            
            // A tag just got assigned  or changed.
            if (dialog.getRemoteTag() == null &&
            transactionResponse.getTo().getTag() != null ) {
                
                // Dont assign tag on provisional response
                if (transactionResponse.getStatusCode() != 100) {
                    dialog.setRemoteTag(transactionResponse.getToTag());
                }
                String dialogId = transactionResponse.getDialogId(false);
                dialog.setDialogId(dialogId);
                if (sipStackImpl.isDialogCreated(method) && 
		    transactionResponse.getStatusCode() != 100 ) {
                    sipStackImpl.putDialog(dialog);
		    if (transactionResponse.getStatusCode()/100 == 1) 
                        dialog.setState(DialogImpl.EARLY_STATE);
		    else if ( transactionResponse.getStatusCode()/100 == 2)
                        dialog.setState(DialogImpl.CONFIRMED_STATE);
                    added = true;
                }
                
            } else if ( dialog.getRemoteTag() 		!= null      &&
            		transactionResponse.getToTag()  != null      &&
            		! dialog.getRemoteTag().equals
            		(transactionResponse.getToTag())) {
                String dialogId = transactionResponse.getDialogId(false);
                dialog.setDialogId(dialogId);
                if (sipStackImpl.isDialogCreated(method)) {
                    sipStackImpl.putDialog(dialog);
                    added = true;
                }
            }
            
            if (sipStackImpl.isDialogCreated(method)) {
                // Make  a final tag assignment.
                if (transactionResponse.getToTag() != null &&
                  transactionResponse.getStatusCode()/100 == 2)  {
                   // This is a dialog creating method (such as INVITE).
                   // 2xx response -- set the state to the confirmed
                  // state.
                    dialog.setRemoteTag(transactionResponse.getToTag());
                    dialog.setState(DialogImpl.CONFIRMED_STATE);
                }  else if (( transactionResponse.getStatusCode()/100 == 4 ||
                         transactionResponse.getStatusCode()/100 == 5 ||
                         transactionResponse.getStatusCode()/100 == 6) &&
                        ( dialog.getState() == null  ||
                          dialog.getState().getValue() ==
                          DialogImpl.EARLY_STATE)) {
                        // Invite transaction generated an error.
                        dialog.setState(DialogImpl.TERMINATED_STATE);
                }
            }
	}
        try {
            if (isInviteTransaction())
                inviteClientTransaction(transactionResponse,sourceChannel);
            else nonInviteClientTransaction(transactionResponse,sourceChannel);
        } catch (IOException ex) {
            setState(TERMINATED_STATE);
            raiseErrorEvent
            ( SIPTransactionErrorEvent.TRANSPORT_ERROR );
        }
    }
    
    /** Implements the state machine for invite client transactions.
     * @param transactionResponse -- transaction response received.
     * @param sourceChannel - source channel on which the response was received.
     *<pre>
     *
     *                                   |Request from TU
     *                                   |send request
     *               Timer E             V
     *               send request  +-----------+
     *                   +---------|           |-------------------+
     *                   |         |  Trying   |  Timer F          |
     *                   +-------->|           |  or Transport Err.|
     *                             +-----------+  inform TU        |
     *                200-699         |  |                         |
     *                resp. to TU     |  |1xx                      |
     *                +---------------+  |resp. to TU              |
     *                |                  |                         |
     *                |   Timer E        V       Timer F           |
     *                |   send req +-----------+ or Transport Err. |
     *                |  +---------|           | inform TU         |
     *                |  |         |Proceeding |------------------>|
     *                |  +-------->|           |-----+             |
     *                |            +-----------+     |1xx          |
     *                |              |      ^        |resp to TU   |
     *                | 200-699      |      +--------+             |
     *                | resp. to TU  |                             |
     *                |              |                             |
     *                |              V                             |
     *                |            +-----------+                   |
     *                |            |           |                   |
     *                |            | Completed |                   |
     *                |            |           |                   |
     *                |            +-----------+                   |
     *                |              ^   |                         |
     *                |              |   | Timer K                 |
     *                +--------------+   | -                       |
     *                                   |                         |
     *                                   V                         |
     *             NOTE:           +-----------+                   |
     *                             |           |                   |
     *         transitions         | Terminated|<------------------+
     *         labeled with        |           |
     *         the event           +-----------+
     *         over the action
     *         to take
     *
     *                 Figure 6: non-INVITE client transaction
     */
    private void nonInviteClientTransaction(
    SIPResponse transactionResponse,
    MessageChannel sourceChannel)
    throws IOException, SIPServerException  {
        int currentState = getState().getValue();
        int statusCode = transactionResponse.getStatusCode();
        if (currentState == TRYING_STATE) {
            if (statusCode / 100 == 1) {
                setState(PROCEEDING_STATE);
                enableRetransmissionTimer
                (MAXIMUM_RETRANSMISSION_TICK_COUNT);
                enableTimeoutTimer(TIMER_F);
            } else if (200 <= statusCode && statusCode <= 699) {
                // Send the response up to the TU.
                respondTo.processResponse( transactionResponse, this );
                if (! isReliable() ) {
                    setState(COMPLETED_STATE);
                    enableTimeoutTimer(TIMER_K);
                } else {
                    setState(TERMINATED_STATE);
                }
            }
        } else  if (currentState == PROCEEDING_STATE &&
        200 <= statusCode && statusCode <= 699) {
            respondTo.processResponse( transactionResponse, this );
            disableRetransmissionTimer( );
            disableTimeoutTimer( );
            if (! isReliable()) {
                setState(COMPLETED_STATE);
                enableTimeoutTimer(TIMER_K);
            } else {
                setState(TERMINATED_STATE);
            }
        }
        
        
    }
    
    
    
    /** Implements the state machine for invite client transactions.
     * @param transactionResponse -- transaction response received.
     * @param sourceChannel - source channel on which the response was received.
     *<pre>
     *
     *                               |INVITE from TU
     *             Timer A fires     |INVITE sent
     *             Reset A,          V                      Timer B fires
     *             INVITE sent +-----------+                or Transport Err.
     *               +---------|           |---------------+inform TU
     *               |         |  Calling  |               |
     *               +-------->|           |-------------->|
     *                         +-----------+ 2xx           |
     *                            |  |       2xx to TU     |
     *                            |  |1xx                  |
     *    300-699 +---------------+  |1xx to TU            |
     *   ACK sent |                  |                     |
     *resp. to TU |  1xx             V                     |
     *            |  1xx to TU  -----------+               |
     *            |  +---------|           |               |
     *            |  |         |Proceeding |-------------->|
     *            |  +-------->|           | 2xx           |
     *            |            +-----------+ 2xx to TU     |
     *            |       300-699    |                     |
     *            |       ACK sent,  |                     |
     *            |       resp. to TU|                     |
     *            |                  |                     |      NOTE:
     *            |  300-699         V                     |
     *            |  ACK sent  +-----------+Transport Err. |  transitions
     *            |  +---------|           |Inform TU      |  labeled with
     *            |  |         | Completed |-------------->|  the event
     *            |  +-------->|           |               |  over the action
     *            |            +-----------+               |  to take
     *            |              ^   |                     |
     *            |              |   | Timer D fires       |
     *            +--------------+   | -                   |
     *                               |                     |
     *                               V                     |
     *                         +-----------+               |
     *                         |           |               |
     *                         | Terminated|<--------------+
     *                         |           |
     *                         +-----------+
     *</pre>
     */
    
    private void inviteClientTransaction(
    SIPResponse transactionResponse,
    MessageChannel sourceChannel) throws IOException, SIPServerException {
        int statusCode = transactionResponse.getStatusCode();
        int currentState = getState().getValue();
        if (currentState == TERMINATED_STATE) {
            // Do nothing in the terminated state.
            return;
        } else if (currentState == CALLING_STATE )  {
            if (statusCode/100 == 2) {
                respondTo.processResponse( transactionResponse, this );
                disableRetransmissionTimer( );
                disableTimeoutTimer( );
                setState(TERMINATED_STATE);
            } else if (statusCode/100 == 1) {
                disableRetransmissionTimer( );
                disableTimeoutTimer( );
                respondTo.processResponse( transactionResponse, this );
                setState(PROCEEDING_STATE);
            } else if (300 <= statusCode  &&  statusCode <= 699) {
                // When in either the "Calling" or "Proceeding" states,
                // reception of response with status code from 300-699
                // MUST cause the client transaction to 
		// transition to "Completed".
                // The client transaction MUST pass the received response up to
                // the TU, and the client transaction MUST generate an 
		// ACK request.
                respondTo.processResponse( transactionResponse, this);
                // Send back an ACK request
                sendMessage(getOriginalRequest(). createAckRequest
                ((To)transactionResponse.getTo()));
                if ( ! isReliable())  {
                    setState(COMPLETED_STATE);
                    enableTimeoutTimer(TIMER_D);
                } else {
                    //Proceed immediately to the TERMINATED state.
                    setState(TERMINATED_STATE);
                }
            }
        } else if (currentState == PROCEEDING_STATE) {
	    if (statusCode / 100 == 1) {
                respondTo.processResponse( transactionResponse, this);
            } else if (statusCode / 100 == 2) {
                setState(TERMINATED_STATE);
                respondTo.processResponse( transactionResponse, this);
                // setState(TERMINATED_STATE);
            } else if (300 <= statusCode  &&  statusCode <= 699) {
                respondTo.processResponse( transactionResponse, this);
                // Send back an ACK request
                sendMessage(getOriginalRequest(). createAckRequest
                ((To)transactionResponse.getTo()));
                if ( ! isReliable())  {
                    setState(COMPLETED_STATE);
                    enableTimeoutTimer(TIMER_D);
                } else {
                    setState(TERMINATED_STATE);
                }
            }
        } else if (currentState == COMPLETED_STATE) {
            if (300 <= statusCode  &&  statusCode <= 699) {
                // Send back an ACK request
                sendMessage(getOriginalRequest(). createAckRequest
                ((To)transactionResponse.getTo()));
            }
            
        }
        
    }
    
    
    
    /**
     * Called by the transaction stack when a retransmission timer
     * fires.
     */
    protected void fireRetransmissionTimer(
    ) {
        
        try {
            // Resend the last request sent
            // System.out.println("fireRetransmissionTimer " );
            getMessageChannel( ).sendMessage( lastRequest );
        } catch( IOException e ) {
            raiseErrorEvent(SIPTransactionErrorEvent.TRANSPORT_ERROR );
        }
        
    }
    
    
    /**
     *	Called by the transaction stack when a timeout timer fires.
     */
    protected void fireTimeoutTimer(
    ) {
        
         DialogImpl dialogImpl = (DialogImpl) this.getDialog();
         if( getState().getValue() == CALLING_STATE ||
            getState().getValue() == TRYING_STATE   ||
            getState().getValue() == PROCEEDING_STATE ) {
            // Timeout occured. If this is asociated with a transaction
            // creation then kill the dialog.
            if (((SIPTransactionStack)getSIPStack()).isDialogCreated
            (this.getOriginalRequest().getMethod())) {
                // terminate the enclosing dialog.
                dialogImpl.setState(DialogImpl.TERMINATED_STATE);
            } else if (getOriginalRequest().getMethod().equalsIgnoreCase
                (Request.BYE)){
                // Terminate the associated dialog on BYE Timeout.
                dialogImpl.setState(DialogImpl.TERMINATED_STATE);
            }
        }
        if ( getState().getValue() != COMPLETED_STATE ) {
            raiseErrorEvent(SIPTransactionErrorEvent.TIMEOUT_ERROR);
        } else {
            setState(TERMINATED_STATE);
        }
        
        
        
    }
    
    
    /** Set the port of the recipient.
     */
    protected void setViaPort(int port) { this.viaPort = port; }
    
    /** Set the port of the recipient.
     */
    protected void setViaHost(String host) { this.viaHost = host; }
    
    /** Get the port of the recipient.
     */
    public int getViaPort() { return this.viaPort; }
    
    /** Get the host of the recipient.
     */
    public String getViaHost() { return this.viaHost; }

    /** get the via header for an outgoing request.
     */
    public Via getOutgoingViaHeader() {
		return this.getMessageProcessor().getViaHeader();
    }

     public boolean isSecure() { return encapsulatedChannel.isSecure(); }
    
}
