package gov.nist.javax.sip.stack;

import gov.nist.javax.sip.message.*;
import gov.nist.javax.sip.parser.*;
import gov.nist.javax.sip.header.*;
import gov.nist.javax.sip.stack.*;
import gov.nist.javax.sip.SIPConstants;
import gov.nist.core.*;
import javax.sip.message.*;
import java.text.ParseException;

import java.io.IOException;


/**
 *	Represents a server transaction.
 *
 *	@author	Jeff Keyser <br/> M. Ranganathan <mranga@nist.gov>
 */
public class SIPServerTransaction
extends SIPTransaction
implements SIPServerRequestInterface, javax.sip.ServerTransaction {
    
    // Real RequestInterface to pass messages to
    private SIPServerRequestInterface	requestOf;

    // Flag to indicate that a cancel has been seen 
    private boolean cancelled;

    
    /**
     *	Creates a new server transaction.
     *
     *	@param newSIPStack Transaction stack this transaction
     * 	belongs to.
     *	@param newChannelToUse Channel to encapsulate.
     */
    protected SIPServerTransaction(
    SIPTransactionStack	newSIPStack,
    MessageChannel		newChannelToUse
    ) {
        
        super( newSIPStack, newChannelToUse );
        if (LogWriter.needsLogging)
            LogWriter.logMessage("Creating Server Transaction" + this);
        
    }
    
    
    /**
     *	Sets the real RequestInterface this transaction encapsulates.
     *
     *	@param newRequestOf RequestInterface to send messages to.
     */
    public void setRequestInterface(
    SIPServerRequestInterface	newRequestOf
    ) {
        
        requestOf = newRequestOf;
        
    }
    
    
    public String getProcessingInfo(
    ) {
        
        return requestOf.getProcessingInfo( );
        
    }
    
    
    /**
     *	Returns this transaction.
     */
    public MessageChannel getResponseChannel(
    ) {
        
        return this;
        
    }
    
    
    
    /**
     *	Deterines if the message is a part of this transaction.
     *
     *	@param messageToTest Message to check if it is part of this
     *		transaction.
     *
     *	@return True if the message is part of this transaction,
     * 		false if not.
     */
    public  boolean isMessagePartOfTransaction(
    SIPMessage	messageToTest ) {
        
        // List of Via headers in the message to test
        ViaList	viaHeaders;
        // Topmost Via header in the list
        Via		topViaHeader;
        // Branch code in the topmost Via header
        String	messageBranch;
        // Flags whether the select message is part of this transaction
        boolean transactionMatches;
        
        
        transactionMatches = false;
	/**
	if (LogWriter.needsLogging)  {
		LogWriter.logMessage("--------- TEST ------------");
		LogWriter.logMessage(" testing " + this.getOriginalRequest());
		LogWriter.logMessage("Against " + messageToTest);
		LogWriter.logMessage("isTerminated = " + isTerminated());
	}
	**/
		
        if( !isTerminated( ) ) {
            
            // Get the topmost Via header and its branch parameter
            viaHeaders = messageToTest.getViaHeaders( );
            if( viaHeaders != null ) {
                
                topViaHeader = (Via)viaHeaders.getFirst( );
                messageBranch = topViaHeader.getBranch( );
                if( messageBranch != null ) {
                    
                    // If the branch parameter exists but
                    // does not start with the magic cookie,
                    if( !messageBranch.startsWith
                    ( SIPConstants.BRANCH_MAGIC_COOKIE ) ) {
                        
                        // Flags this as old
                        // (RFC2543-compatible) client
                        // version
                        messageBranch = null;
                        
                    }
                    
                }
                
                // If a new branch parameter exists,
                if( messageBranch != null  &&
                this.getBranch() != null ) {
                    
                    if( getBranch().equals(messageBranch)
                    && topViaHeader.getSentBy( ).
                    equals(((Via)getOriginalRequest().
                    getViaHeaders().getFirst()).
                    getSentBy())
                    ) {
			// Matching server side transaction with only the
			// branch parameter.
                        transactionMatches = true;
                    }
                    
                    // If this is an RFC2543-compliant message,
                } else {
                    
                    // If RequestURI, To tag, From tag,
                    // CallID, CSeq number, and top Via
                    // headers are the same,
                    String originalFromTag =
                    getOriginalRequest().getFrom().
                    getTag();
                    
                    String thisFromTag =
                    messageToTest.getFrom().getTag();
                    
                    boolean skipFrom =
                    ( originalFromTag == null ||
                    thisFromTag == null);
                    
                    
                    String originalToTag =
                    getOriginalRequest().getTo().
                    getTag();
                    
                    String thisToTag =
                    messageToTest.getTo().getTag();
                    
                    boolean skipTo =
                    (originalToTag == null ||
                    thisToTag == null);
                    
                    
                    
                    if( getOriginalRequest().
                    getRequestURI().
                    equals( ( (SIPRequest)messageToTest ).
                    getRequestURI()) &&
                    
                    ( skipFrom ||
                    originalFromTag.equals(thisFromTag) ) &&
                    
                    ( skipTo ||
                    originalToTag.equals(thisToTag) ) &&
                    
                    
                    getOriginalRequest( ).
                    getCallId( ).getCallId( ).
                    equals( messageToTest.getCallId( )
                    .getCallId( ) ) &&
                    getOriginalRequest( ).
                    getCSeq( ).getSequenceNumber( ) ==
                    messageToTest.getCSeq( ).
                    getSequenceNumber( ) &&
                    
                    topViaHeader.equals(
                    getOriginalRequest( ).
                    getViaHeaders().getFirst() ) ) {
                        
                        transactionMatches = true;
                    }
                    
                }
                
            }
            
        }
        return transactionMatches;
        
    }
    
    
    /**
     *	Process a new request message through this transaction.
     * If necessary, this message will also be passed onto the TU.
     *
     *	@param transactionRequest Request to process.
     *	@param sourceChannel Channel that received this message.
     */
    public void processRequest(
    SIPRequest		transactionRequest,
    MessageChannel	sourceChannel
    ) throws SIPServerException {
        
        try {
            
            // If this is the first request for this transaction,
            if( getState( )== null) {
                
                // Save this request as the one this
                // transaction is handling
                setOriginalRequest( transactionRequest );
                setState( TRYING_STATE );
                if( isInviteTransaction( ) ) {
                    
                    // Has side-effect of setting
                    // state to "Proceeding"
                    sendMessage( transactionRequest.
                    createResponse( 100, "Trying" ) );
                    
                } 
                
                // If an invite transaction is ACK'ed while in
                // the completed state,
            } else if( isInviteTransaction( ) &&
            getState().getValue() == COMPLETED_STATE &&
            transactionRequest.getMethod( ).equals( Request.ACK ) ) {
                
                setState( CONFIRMED_STATE );
                disableRetransmissionTimer( );
                if( !isReliable( ) ) {
                    
                    enableTimeoutTimer( TIMER_I );
                    
                } else {
                    
                    setState( TERMINATED_STATE );
                    
                }
                
                // If we receive a retransmission of the original
                // request,
            } else if( transactionRequest.getMethod( ).equals
            ( getOriginalRequest( ).getMethod( ) ) ) {
                
                if( getState( ).getValue() == PROCEEDING_STATE ||
                getState( ).getValue() == COMPLETED_STATE ) {
                    
                    // Resend the last response to
                    // the client
                    if (lastResponse != null)  {
        		try {
            		   // Send the message to the client
            		   getMessageChannel( ).sendMessage
					( lastResponse );
        		} catch( IOException e ) {
            			setState( TERMINATED_STATE );
            			throw e;
            
        		}
		   }
                } else if (transactionRequest.getMethod().
                equals(Request.ACK) ) {
                    // This is passed up to the TU to suppress
                    // retransmission of OK
                    requestOf.processRequest
                    ( transactionRequest, this );
                }
                return;
                
            }
            
            // Pass message to the TU
	    if (getState().getValue() != COMPLETED_STATE && 
		getState().getValue() != TERMINATED_STATE ) 
                requestOf.processRequest( transactionRequest, this );
            
        } catch( IOException e ) {
            raiseErrorEvent
            ( SIPTransactionErrorEvent.TRANSPORT_ERROR );
        }
        
    }
    
    
    /**
     *	Send a response message through this transactionand onto
     *      the client.
     *
     *	@param messageToSend Response to process and send.
     */
    public void sendMessage(
    SIPMessage	messageToSend
    ) throws IOException {
        
        // Message typecast as a response
        SIPResponse	transactionResponse;
        // Status code of the response being sent to the client
        int			statusCode;
        
        
        // Get the status code from the response
        transactionResponse = (SIPResponse)messageToSend;
        statusCode = transactionResponse.getStatusCode( );

	DialogImpl dialog = this.dialog;

	// Transaction successfully cancelled but dialog has not yet
	// been established so delete the dialog.

	if (transactionResponse.getCSeq().getMethod().equalsIgnoreCase
	    ("CANCEL") && transactionResponse.getStatusCode() == 200 &&
	     ((SIPTransactionStack)getSIPStack()).
		isDialogCreated(getOriginalRequest().getMethod()) &&
	     ( dialog.getState() == null ||
	       dialog.getState().getValue() == DialogImpl.EARLY_STATE )) {
	       dialog.setState(DialogImpl.TERMINATED_STATE);
	}

		
        
        // super.checkCancel(transactionResponse);

	try {
	// Provided we have set the banch id for this we set the BID for the
	// outgoing via.
	if (this.getBranch() != null) 
	    transactionResponse.getTopmostVia().setBranch(this.getBranch());
	else 
	    transactionResponse.getTopmostVia().removeParameter
				(ParameterNames.BRANCH);
	} catch (ParseException ex) { 	
		ex.printStackTrace();
	}

	// Put a dialog record in the SIP Stack if necessary.
	if (this.dialog != null &&
	    this.dialog.getRemoteTag() == null &&
	    transactionResponse.getTo().getTag() != null  &&
	    ((SIPTransactionStack) this.getSIPStack()).isDialogCreated
	    (transactionResponse.getCSeq().getMethod())) {
	    this.dialog.setRemoteTag(transactionResponse.getTo().getTag());
	    ((SIPTransactionStack) this.getSIPStack()).putDialog(this.dialog);
	    if (statusCode/100 == 1) 
			this.dialog.setState(DialogImpl.EARLY_STATE);
	} else if (((SIPTransactionStack) this.getSIPStack()).isDialogCreated
	    (transactionResponse.getCSeq().getMethod())) {
            if (statusCode / 100 == 2)  {
		this.dialog.setState(DialogImpl.CONFIRMED_STATE);
	    } else if (statusCode /100 >= 3  && statusCode / 100 <=6)  {
		this.dialog.setState(DialogImpl.TERMINATED_STATE);
	    }
	} else if (transactionResponse.getCSeq().getMethod().equals
			(Request.BYE) && statusCode/100 == 2 ) {
		// Dialog will be terminated when the transction is terminated.
		if ( ! isReliable())
		   this.dialog.setState(DialogImpl.COMPLETED_STATE);
		else 
		   this.dialog.setState(DialogImpl.TERMINATED_STATE);
	}
	
        
        
        // If the TU sends a provisional response while in the
        // trying state,
	if (  getState().getValue() ==  TRYING_STATE  )   {
	   if ( statusCode / 100 == 1 ) {
               setState( PROCEEDING_STATE );
	    } else if (  200 <= statusCode  && statusCode <= 699) {
		 setState (COMPLETED_STATE);
                 if( !isReliable( ) ) {
                        
                        enableRetransmissionTimer( );
                        
                  } 
                  enableTimeoutTimer( TIMER_J );
	    }
            
            // If the transaction is in the proceeding state,
        } else if( getState( ).getValue() == PROCEEDING_STATE ) {
            
            if( isInviteTransaction( ) ) {
                
                // If the response is a failure message,
		if (statusCode / 100 == 2 && 
			((SIPTransactionStack) getSIPStack()).
			retransmissionFilter ) {
                    // Set up to catch returning ACKs
                    setState( TERMINATED_STATE );
                    if( !isReliable( ) ) {
                        
                        enableRetransmissionTimer( );
                        
                    }
                    enableTimeoutTimer( TIMER_J );
                } else if(300 <= statusCode && statusCode <= 699) { 
                    
                    // Set up to catch returning ACKs
                    setState( COMPLETED_STATE );
                    if( !isReliable( ) ) {
                        
                        enableRetransmissionTimer( );
                        
                    } 
                    enableTimeoutTimer( TIMER_J );
                    
                    // If the response is a success message,
                } else if( statusCode / 100 == 2 ) {
                    
                    // Terminate the transaction
                    setState( TERMINATED_STATE );
                    disableRetransmissionTimer( );
                    disableTimeoutTimer( );
                    
                }
                
                // If the transaction is not an invite transaction
                // and this is a final response,
            } else if( 200 <= statusCode && statusCode <= 699 ) {
                
                // Set up to retransmit this response,
                // or terminate the transaction
                setState( COMPLETED_STATE );
                if( !isReliable( ) ) {
                    
		    disableRetransmissionTimer();
                    enableTimeoutTimer( TIMER_J );
                    
                } else {
                    
                    setState( TERMINATED_STATE );
                    
                }
                
            }
            
            // If the transaction has already completed,
        } else if( getState( ).getValue() == COMPLETED_STATE ) {
            
            return;
        }
        try {
            
            // Send the message to the client
            lastResponse = transactionResponse;
            getMessageChannel( ).sendMessage( transactionResponse );
            
        } catch( IOException e ) {
            
            setState( TERMINATED_STATE );
            throw e;
            
        }
        
    }
    
    public String getViaHost(
    ) {
        
        return encapsulatedChannel.getViaHost( );
        
    }
    
    
    public int getViaPort() {
        
        return encapsulatedChannel.getViaPort( );
        
    }
    
    /**
     *	Called by the transaction stack when a retransmission
     *  timer fires. This retransmits the last response when the
     *  retransmission filter is enabled.
     */
    protected  void fireRetransmissionTimer(
    ) {
        
        try {
            
            // Resend the last response sent by this transaction
	    if (isInviteTransaction() && 
	        ((SIPTransactionStack)getSIPStack()).retransmissionFilter ) 
                getMessageChannel( ).sendMessage( lastResponse );
            
        } catch( IOException e ) {
            raiseErrorEvent
            ( SIPTransactionErrorEvent.TRANSPORT_ERROR );
            
        }
        
    }
    
    
    /**
     *	Called by the transaction stack when a timeout timer fires.
     */
    protected void fireTimeoutTimer(
    ) {

	System.out.println("SIPServerTransaction.fireTimeoutTimer " 
		+ this.getState() + " method = " + 
		this.getOriginalRequest().getMethod() );

        DialogImpl dialog = (DialogImpl) this.getDialog();
        int mystate = this.getState().getValue();
	if (((SIPTransactionStack)getSIPStack()).isDialogCreated
	       (this.getOriginalRequest().getMethod()) && 
               ( mystate == super.CALLING_STATE ||
                 mystate == super.TRYING_STATE )) {
	        dialog.setState(DialogImpl.TERMINATED_STATE);
         } else if (getOriginalRequest().getMethod().equals(Request.BYE)){
             if (dialog != null) 
                    dialog.setState(DialogImpl.TERMINATED_STATE);
         }

        if( getState( ).getValue() == CONFIRMED_STATE &&
            isInviteTransaction() ) {
            
            raiseErrorEvent
            ( SIPTransactionErrorEvent.TIMEOUT_ERROR );
	    setState( TERMINATED_STATE);
            
        } else if(  ! isInviteTransaction() && (
	   getState( ).getValue() == COMPLETED_STATE  ||
	  getState().getValue() == CONFIRMED_STATE ) ) {
            
            setState( TERMINATED_STATE );
            
        }
        
    }
    
    /** Get the last response.
     */
    public SIPResponse getLastResponse() {
        return this.lastResponse;
    }

    /** Set the original request.
     */
     public void setOriginalRequest(SIPRequest originalRequest) {
		super.setOriginalRequest(originalRequest);
		// ACK Server Transaction is just a dummy transaction.
		if (originalRequest.getMethod().equals("ACK"))  
			this.setState(TERMINATED_STATE);

     }


    

    
}
