package gov.nist.javax.sip.stack;
import gov.nist.javax.sip.parser.*;
import gov.nist.javax.sip.message.*;
import gov.nist.core.*;
import javax.sip.message.*;
import javax.sip.*;
import javax.sip.address.*;

import java.util.*;
import java.io.IOException;
import java.net.*;


/**
 *	Adds a transaction layer to the {@link SIPStack} class.  This is done by
 * replacing the normal MessageChannels returned by the base class with
 * transaction-aware MessageChannels that encapsulate the original channels
 * and handle the transaction state machine, retransmissions, etc.
 *
 *@author	Jeff Keyser (original) <br/>
 * M. Ranganathan <mranga@nist.gov>  <br/> (Added Dialog table).
 *<a href="{@docRoot}/uncopyright.html">This code is in the public domain.</a>
 *
 *@version  JAIN-SIP-1.1
 *
 */
public abstract class SIPTransactionStack
extends SIPStack implements  SIPTransactionEventListener {
    
    /**
     * Number of milliseconds between timer ticks (500).
     **/
    public static final int	BASE_TIMER_INTERVAL = 500;
    
    // Collection of current client transactions
    private Set		clientTransactions;
    // Collection or current server transactions
    private Set		serverTransactions;
    // Table of dialogs.
    private Hashtable	dialogTable;
    
    // Max number of server transactions concurrent.
    protected int transactionTableSize;
    
    // A table of assigned dialogs.
    
    
    // Retransmission filter - indicates the stack will retransmit 200 OK
    // for invite transactions.
    protected boolean retransmissionFilter;
    
    // A set of methods that result in dialog creations.
    protected HashSet dialogCreatingMethods;
    
    
    
    
    /**
     *	Default constructor.
     */
    protected SIPTransactionStack() {
        super();
        this.transactionTableSize = -1;
        // a set of methods that result in dialog creation.
        this.dialogCreatingMethods = new HashSet();
        // Standard set of methods that create dialogs.
        this.dialogCreatingMethods.add(Request.INVITE);
        this.dialogCreatingMethods.add(Request.SUBSCRIBE);
        this.dialogCreatingMethods.add(Request.NOTIFY);
        this.dialogCreatingMethods.add(Request.MESSAGE);
        // Create the transaction collections
        clientTransactions =
        Collections.synchronizedSet( new HashSet( ) );
        serverTransactions =
        Collections.synchronizedSet( new HashSet( ) );
        // Dialog dable.
        this.dialogTable = new Hashtable();
        
        
        // Start the timer event thread.
        System.out.println("Starting timeout");
        new Thread( new TransactionScanner() ).start( );
        
    }
    /** Return true if extension is supported.
     *
     *@return true if extension is supported and false otherwise.
     *
     */
    public boolean isDialogCreated(String method) {
        return dialogCreatingMethods.contains(method.toUpperCase());
    }
    
    
    /** Add an extension method.
     *
     *@param extensionMethod -- extension method to support for dialog
     * creation
     */
    public void addExtensionMethod(String extensionMethod) {
        this.dialogCreatingMethods.add(extensionMethod);
    }
    
    /** Put a dialog into the dialog table.
     *
     *@param dialog -- dialog to put into the dialog table.
     *
     */
    
    public void
    putDialog(DialogImpl dialog) {
        String dialogId  = dialog.getDialogId();
        if (LogWriter.needsLogging) {
            LogWriter.logMessage("putDialog dialogId=" + dialogId);
        }
        if (this.getDefaultRouteHeader() != null)
            dialog.addRoute(this.getDefaultRouteHeader(),false);
        dialog.setStack(this);
        if (LogWriter.needsLogging)
            LogWriter.logStackTrace();
        synchronized(dialogTable) {
            dialogTable.put(dialogId,dialog);
        }
        
    }
    
    public synchronized DialogImpl
    createDialog(SIPTransaction transaction) {
        SIPRequest sipRequest = transaction.getOriginalRequest();
        
        DialogImpl retval = new DialogImpl(transaction);
        
        return retval;
        
    }
    
    
    
    /** Return the dialog for a given dialog ID. If compatibility is
     * enabled then we do not assume the presence of tags and hence
     * need to add a flag to indicate whether this is a server or
     * client transaction.
     *
     *@param dialogId is the dialog id to check.
     *
     */
    
    public DialogImpl
    getDialog(String dialogId) {
	if (LogWriter.needsLogging)
		LogWriter.logMessage("Getting dialog for " + dialogId);
        synchronized(dialogTable) {
            return (DialogImpl)dialogTable.get(dialogId);
        }
    }
    
    
    /** Find the transaction corresponding to a given request.
     *
     *@param sipRequest -- request for which to retrieve the transaction.
     *
     *@param isServer -- search the server transaction table if true.
     *
     *@return the transaction object corresponding to the request or null
     * if no such mapping exists.
     */
    public SIPTransaction
    findTransaction(SIPMessage sipMessage, boolean isServer) {
        if (isServer) {
            Iterator it = serverTransactions.iterator();
            while (it.hasNext()) {
                SIPServerTransaction sipServerTransaction =
                (SIPServerTransaction)it.next();
                if (sipServerTransaction.isMessagePartOfTransaction(sipMessage))
                    return sipServerTransaction;
            }
        } else {
            Iterator it = clientTransactions.iterator();
            while (it.hasNext()) {
                SIPClientTransaction clientTransaction =
                (SIPClientTransaction)it.next();
                if (clientTransaction.isMessagePartOfTransaction(sipMessage))
                    return clientTransaction;
            }
            
        }
        return null;
        
        
    }
    
    /** Get the transaction to cancel. Search the server transaction
     * table for a transaction that matches the given transaction.
     */
    public SIPTransaction
    findCancelTransaction(SIPRequest cancelRequest, boolean isServer) {

	if (LogWriter.needsLogging) {
	   LogWriter.logMessage("findCancelTransaction request= \n" 
			+ cancelRequest  +
		"\nfindCancelRequest isServer=" + isServer  );
	}

        if (isServer) {
            synchronized (this.serverTransactions) {
                Iterator li = this.serverTransactions.iterator();
                while(li.hasNext()) {
                    SIPTransaction transaction = (SIPTransaction)li.next();
                    SIPRequest sipRequest =
                    (SIPRequest) (transaction.getRequest());
                    
                    SIPServerTransaction sipServerTransaction =
                    (SIPServerTransaction) transaction;
                    if (sipServerTransaction.doesCancelMatchTransaction
                    (cancelRequest))
                        return sipServerTransaction;
                }
            }
        } else{
            synchronized (this.clientTransactions) {
                Iterator li = this.clientTransactions.iterator();
                while(li.hasNext()) {
                    SIPTransaction transaction = (SIPTransaction)li.next();
                    SIPRequest sipRequest =
                    (SIPRequest) (transaction.getRequest());
                    
                    SIPClientTransaction sipClientTransaction =
                    (SIPClientTransaction) transaction;
                    if (sipClientTransaction.doesCancelMatchTransaction
                    (cancelRequest))
                        return sipClientTransaction;
                    
                }
            }
        }
        return null;
    }
    
    
    
    
    /**
     * Construcor for the stack. Registers the request and response
     * factories for the stack.
     *
     * @param messageFactory User-implemented factory for processing
     *	messages.
     */
    protected SIPTransactionStack(
    SIPStackMessageFactory	messageFactory
    ) {
        this();
        super.sipMessageFactory = messageFactory;
    }
    
    
    
    /**
     *	Thread used to throw timer events for all transactions.
     */
    class TransactionScanner implements Runnable {
        public void run() {
            
            // Iterator through all transactions
            Iterator		transactionIterator;
            // One transaction in the set
            SIPTransaction	nextTransaction;
            
            
            // Loop while this stack is running
            while( isAlive( ) ) {
                
                try {
                    
                    // Sleep for one timer "tick"
                    Thread.sleep( BASE_TIMER_INTERVAL );

		    //  System.out.println("clientTransactionTable size " +
	 	    // 	clientTransactions.size());
		    // System.out.println("serverTransactionTable size " + 
		    // serverTransactions.size());
                    // Check all client transactions

                    LinkedList fireList = new LinkedList();
                    
                    // Check all server transactions
                    synchronized(serverTransactions) {
                        transactionIterator =
                        serverTransactions.iterator( );
                        while( transactionIterator.hasNext( ) ) {
                            
                            nextTransaction =
                            (SIPTransaction)
                            transactionIterator.next( );
                            
                            // If the transaction has terminated,
                            if( nextTransaction.isTerminated( ) ) {
                                
                                // Remove it from the set
				if (LogWriter.needsLogging)
				    LogWriter.logMessage("removing" +
					nextTransaction );
                                transactionIterator.remove( );
                                
                                // If this transaction has not
                                //terminated,
                            } else {
                                // Add to the fire list -- needs to be moved
                                // outside the synchronized block to prevent
                                // deadlock.
				/**
				System.out.println("state = " +
					nextTransaction.getState() + "/" +
					nextTransaction.getOriginalRequest().
					getMethod());
				**/
                                fireList.add(nextTransaction);
                                
                            }
                            
                        }
                    }
                    
                    
                    synchronized(clientTransactions) {
                        transactionIterator =
                        clientTransactions.iterator( );
                        while( transactionIterator.hasNext( ) ) {
                            
                            nextTransaction = (SIPTransaction)
                            transactionIterator.next( );
                            
                            // If the transaction has terminated,
                            if( nextTransaction.isTerminated( ) ) {
                                
                                // Remove it from the set
                                if (LogWriter.needsLogging) {
                                    LogWriter.logMessage
                                    ("Removing clientTransaction " +
                                    nextTransaction);
                                }
                                transactionIterator.remove( );
                                
                                // If this transaction has not
                                // terminated,
                            } else {
                                // Add to the fire list -- needs to be moved
                                // outside the synchronized block to prevent
                                // deadlock. 
                                fireList.add(nextTransaction);
                                
			   }
                        }
                    }
                    
                    synchronized (dialogTable) {
                        Collection values = dialogTable.values();
                        Iterator iterator = values.iterator();
                        while (iterator.hasNext()) {
                            DialogImpl d = (DialogImpl) iterator.next();
			    // System.out.println("dialogState = " +
			    //	d.getState() + 
			    //	" isServer = " +  d.isServer());
                            if ( d.getState() != null  &&
                            d.getState().getValue() ==
                            DialogImpl.TERMINATED_STATE ) {
                                if (LogWriter.needsLogging) {
                                    String dialogId =
                                    d.getDialogId();
                                    LogWriter.logMessage
                                    ("Removing Dialog " +
                                    dialogId);
                                }
                                iterator.remove();
                            }
                        }
                    }
                    
                    // Clean up the assigned dialogs table.
                    
                    
                    transactionIterator = fireList.iterator();
                    while (transactionIterator.hasNext()) {
                        nextTransaction =
                        (SIPTransaction)
                        transactionIterator.next();
                        nextTransaction.fireTimer();
                    }
                    
                    
                } catch( InterruptedException e ) {
                    
                    // Ignore
                    
                }
                
            }
            
        }
    }
    
    
    /**
     *	Handles a new SIP request.
     * It finds a server transaction to handle
     * this message.  If none exists, it creates a new transaction.
     *
     *	@param requestReceived		Request to handle.
     *	@param requestMessageChannel	Channel that received message.
     *
     *	@return A server transaction.
     */
    protected SIPServerRequestInterface newSIPServerRequest(
    SIPRequest		requestReceived,
    MessageChannel	requestMessageChannel
    ) {
        
        // Iterator through all server transactions
        Iterator				transactionIterator;
        // Next transaction in the set
        SIPServerTransaction	nextTransaction;
        // Transaction to handle this request
        SIPServerTransaction	currentTransaction;
        
        
        // Loop through all server transactions
        synchronized(serverTransactions) {
            transactionIterator = serverTransactions.iterator( );
            currentTransaction = null;
            while( transactionIterator.hasNext( ) &&
            currentTransaction == null ) {
                
                nextTransaction =
                (SIPServerTransaction)transactionIterator.next( );
                
                // If this transaction should handle this request,
                if( nextTransaction.isMessagePartOfTransaction
                ( requestReceived ) ) {
                    // Mark this transaction as the one
                    // to handle this message
                    currentTransaction = nextTransaction;
                    
                }
                
            }

	    
            
            // If no transaction exists to handle this message,
            if( currentTransaction == null ) {
		if (transactionTableSize == -1 ||
		    serverTransactions.size() < transactionTableSize) {
                   // Create a new server transaction
                   currentTransaction =
                    createServerTransaction(requestMessageChannel);
                   // Need to set this here -- otherwise race condition!!
                   currentTransaction.setOriginalRequest(requestReceived);
                   serverTransactions.add( currentTransaction );
		} else 
		   if (serverTransactions.size() >= transactionTableSize) {
		   // Table overflow so close the message channel caller
		     if (LogWriter.needsLogging) 
			LogWriter.logMessage
			("Could not create server transaction table overflow");
		     requestMessageChannel.close();
		     return null;
		}
                
            }
            
            // Set ths transaction's encapsulated request
            // interface from the superclass
            currentTransaction.setRequestInterface
            ( super.newSIPServerRequest
            ( requestReceived, currentTransaction ) );
            return currentTransaction;
        }
        
    }
    
    
    /**
     *	Handles a new SIP response.
     * It finds a client transaction to handle
     * this message.  If none exists, it sends the message directly to the
     * superclass.
     *
     *	@param responseReceived			Response to handle.
     *	@param responseMessageChannel	Channel that received message.
     *
     *	@return A client transaction.
     */
    protected  SIPServerResponseInterface newSIPServerResponse(
    SIPResponse		responseReceived,
    MessageChannel	responseMessageChannel
    ) {
        //	System.out.println("response = " + responseReceived.encode());
        
        // Iterator through all client transactions
        Iterator				transactionIterator;
        // Next transaction in the set
        SIPClientTransaction	nextTransaction;
        // Transaction to handle this request
        SIPClientTransaction	currentTransaction;
        
        
        // Loop through all server transactions
        synchronized(clientTransactions) {
            transactionIterator = clientTransactions.iterator( );
            currentTransaction = null;
            while( transactionIterator.hasNext() &&
            currentTransaction == null ) {
                
                nextTransaction =
                (SIPClientTransaction)transactionIterator.next( );
                
                // If this transaction should handle this request,
                if( nextTransaction.isMessagePartOfTransaction
                ( responseReceived ) ) {
                    
                    // Mark this transaction as the one to
                    // handle this message
                    currentTransaction = nextTransaction;
                    
                }
                
            }
        }
        
        // If no transaction exists to handle this message,
        if( currentTransaction == null ) {
            
            // Pass the message directly to the TU
            return super.newSIPServerResponse
            ( responseReceived, responseMessageChannel );
            
        }
        
        // Set ths transaction's encapsulated response interface
        // from the superclass
        currentTransaction.setResponseInterface
        ( super.newSIPServerResponse
        ( responseReceived, currentTransaction ) );
        return currentTransaction;
        
    }
    
    
    /**
     *	Creates a client transaction to handle a new request.
     * Gets the real
     * message channel from the superclass, and then creates a new client
     * transaction wrapped around this channel.
     *
     *	@param nextHop	Hop to create a channel to contact.
     */
    public MessageChannel createMessageChannel(
    Hop		nextHop
    ) throws UnknownHostException {
        synchronized (clientTransactions) {
            // New client transaction to return
            SIPTransaction	returnChannel;
            
            
            // Create a new client transaction around the
            // superclass' message channel
            returnChannel =
            createClientTransaction
            ( super.createMessageChannel( nextHop ) );
            clientTransactions.add( returnChannel );
            ((SIPClientTransaction)returnChannel).setViaPort(nextHop.getPort());
            ((SIPClientTransaction)returnChannel).setViaHost(nextHop.getHost());
            return returnChannel;
        }
        
    }
    
    
    /** Create a client transaction from a raw channel.
     *
     *@param rawChannel is the transport channel to encapsulate.
     */
    
    
    public MessageChannel createMessageChannel
    (MessageChannel rawChannel) {
        synchronized(clientTransactions) {
            // New client transaction to return
            SIPTransaction	returnChannel =
            createClientTransaction(rawChannel);
            clientTransactions.add(returnChannel);
            ((SIPClientTransaction)returnChannel).setViaPort
            (rawChannel.getViaPort());
            ((SIPClientTransaction)returnChannel).setViaHost
            (rawChannel.getHost());
            return returnChannel;
        }
    }
    
    /** Create a client transaction from a raw channel.
     *
     *@param rawChannel is the transport channel to encapsulate.
     */
    
    
    public MessageChannel createMessageChannel
    (SIPTransaction transaction) {
        synchronized(clientTransactions) {
            // New client transaction to return
            SIPTransaction	returnChannel =
            createClientTransaction(transaction.getMessageChannel());
            clientTransactions.add(returnChannel);
            ((SIPClientTransaction)returnChannel).setViaPort
            (transaction.getViaPort());
            ((SIPClientTransaction)returnChannel).setViaHost
            (transaction.getViaHost());
            return returnChannel;
        }
    }
    
    
    
    /**
     *	Creates a client transaction that encapsulates a MessageChannel.
     * Useful for implementations that want to subclass the standard
     *
     *	@param encapsulatedMessageChannel
     * 		Message channel of the transport layer.
     */
    public SIPClientTransaction createClientTransaction(
    MessageChannel	encapsulatedMessageChannel
    ) {
        
        return new SIPClientTransaction
        ( this, encapsulatedMessageChannel );
        
    }
    
    
    /**
     *	Creates a server transaction that encapsulates a MessageChannel.
     * Useful for implementations that want to subclass the standard
     *
     *	@param encapsulatedMessageChannel
     * 		Message channel of the transport layer.
     */
    public SIPServerTransaction createServerTransaction(
    MessageChannel	encapsulatedMessageChannel
    ) {
        
        return new SIPServerTransaction
        ( this, encapsulatedMessageChannel );
        
    }
    
    /** Creates a raw message channel. A raw message channel has no
     * transaction wrapper.
     *
     *@param hop -- hop for which to create the raw message channel.
     *
     */
    public MessageChannel createRawMessageChannel(Hop hop)
    throws UnknownHostException {
        return super.createMessageChannel(hop);
    }


   /** Add a new client transaction to the set of existing transactions.
    *
    *@param clientTransaction -- client transaction to add to the set.
    */

    public  void addTransaction
		(SIPClientTransaction clientTransaction) {
	synchronized (clientTransactions) {
		clientTransactions.add(clientTransaction);
	}
    }
    
    
}
