/**
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Unpublished - rights reserved under the Copyright Laws of the United States.
 * Copyright  2003 Sun Microsystems, Inc. All rights reserved.
 *
 * U.S. Government Rights - Commercial software. Government users are subject 
 * to the Sun Microsystems, Inc. standard license agreement and applicable 
 * provisions of the FAR and its supplements.
 *
 * Use is subject to license terms.
 *
 * This distribution may include materials developed by third parties. Sun, 
 * Sun Microsystems, the Sun logo, Java, Jini and JAIN are trademarks or 
 * registered trademarks of Sun Microsystems, Inc. in the U.S. and other 
 * countries.
 *
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 *
 * Module Name   : JAIN SIP Specification
 * File Name     : MessageFactory.java
 * Author        : Phelim O'Doherty
 *
 *  HISTORY
 *  Version   Date      Author              Comments
 *  1.1     08/10/2002  Phelim O'Doherty    Initial version
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package javax.sip.message;

import java.text.ParseException;
import javax.sip.header.*;
import javax.sip.address.URI;
import java.util.List;

/**
 * This interface provides factory methods that allow an application to create 
 * Request and Response messages from a particular implementation of JAIN SIP.
 * This class is a singleton and can be retrieved from the 
 * {@link javax.sip.SipFactory#createMessageFactory()}.
 *
 * @version 1.1
 * @author Sun Microsystems
 */
public interface MessageFactory {

// Standard Request Creation methods

    /**
     * Creates a new Request message of type specified by the method paramater,
     * containing the URI of the Request, the mandatory headers of the message
     * with a body in the form of a Java object and the body content type.
     *
     * @param requestURI the new URI object of the requestURI value of this Message.
     * @param method the new string of the method value of this Message.
     * @param callId the new CallIdHeader object of the callId value of this Message.
     * @param cSeq the new CSeqHeader object of the cSeq value of this Message.
     * @param from the new FromHeader object of the from value of this Message.
     * @param to the new ToHeader object of the to value of this Message.
     * @param via the new List object of the ViaHeaders of this Message.
     * @param contentType the new ContentTypeHeader object of the content type 
     * value of this Message.
     * @param content the new Object of the body content value of this Message.
     * @throws ParseException which signals that an error has been reached
     * unexpectedly while parsing the method or the body.
     */
    public Request createRequest(URI requestURI, String method, CallIdHeader 
            callId, CSeqHeader cSeq, FromHeader from, ToHeader to, List via,
                MaxForwardsHeader maxForwards, ContentTypeHeader contentType,
                Object content) throws ParseException;

    /**
     * Creates a new Request message of type specified by the method paramater,
     * containing the URI of the Request, the mandatory headers of the message
     * with a body in the form of a byte array and body content type.
     *
     * @param requestURI the new URI object of the requestURI value of this Message.
     * @param method the new string of the method value of this Message.
     * @param callId the new CallIdHeader object of the callId value of this Message.
     * @param cSeq the new CSeqHeader object of the cSeq value of this Message.
     * @param from the new FromHeader object of the from value of this Message.
     * @param to the new ToHeader object of the to value of this Message.
     * @param via the new List object of the ViaHeaders of this Message.
     * @param contentType the new ContentTypeHeader object of the content type 
     * value of this Message.
     * @param content the new byte array of the body content value of this Message.
     * @throws ParseException which signals that an error has been reached
     * unexpectedly while parsing the method or the body.
     */
    public Request createRequest(URI requestURI, String method, CallIdHeader 
            callId, CSeqHeader cSeq, FromHeader from, ToHeader to, List via,
                MaxForwardsHeader maxForwards, ContentTypeHeader contentType, 
                byte[] content) throws ParseException;

    /**
     * Creates a new Request message of type specified by the method paramater,
     * containing the URI of the Request, the mandatory headers of the message.
     * This new Request does not contain a body.
     *
     * @param requestURI the new URI object of the requestURI value of this Message.
     * @param method the new string of the method value of this Message.
     * @param callId the new CallIdHeader object of the callId value of this Message.
     * @param cSeq the new CSeqHeader object of the cSeq value of this Message.
     * @param from the new FromHeader object of the from value of this Message.
     * @param to the new ToHeader object of the to value of this Message.
     * @param via the new List object of the ViaHeaders of this Message.
     * @throws ParseException which signals that an error has been reached
     * unexpectedly while parsing the method.
     */
    public Request createRequest(URI requestURI, String method, CallIdHeader 
            callId, CSeqHeader cSeq, FromHeader from, ToHeader to, List via,
                MaxForwardsHeader maxForwards) throws ParseException;

    /**
     * Creates a new BYE Request message based on the dialog creating request 
     * specified by the request paramater (for example, INVITE). 
     *
     * @param request the dialog creating request, the dialog of which this 
     * BYE will terminate.
     * @param isServer the boolean that determines if the dialog request was a 
     * client or server initiated request.
     */
    public Request createByeRequest(Request request, boolean isServer);
    

    /**
     * Creates a new CANCEL Request message based on the dialog creating 
     * request specified by the request paramater (for example, INVITE).
     * The CANCEL request, is used to cancel a previous request sent by a
     * client. Specifically, it asks the UAS to cease processing the request 
     * and to generate an error response to that request. A CANCEL request 
     * constitutes its own transaction, but also references the transaction to 
     * be cancelled. CANCEL has no effect on a request to which a UAS has 
     * already given a final response.
     * <p>
     * Note that both the transaction corresponding to the original request and
     * the CANCEL transaction will complete independently.  However, a UAC
     * canceling a request cannot rely on receiving a 487 (Request Terminated)
     * response for the original request, as an RFC 2543 compliant UAS will
     * not generate such a response. Therefore if there is no final response for
     * the original request the application will receieve a TimeoutEvent with
     * {@link javax.sip.Timeout#TRANSACTION} and the client should then consider the
     * original transaction cancelled.
     * <ul>
     * <li> UAC - A UAC should not send a CANCEL request to any request explicitly
     * supported by this specification other than INVITE request. The reason 
     * being requests other than INVITE are responded to immediately and sending 
     * a CANCEL for a non-INVITE request would always create a race condition. 
     * CANCELs are useful as a UAC can not send a BYE request on a dialogue 
     * until receipt of 2xx final response to the INVITE request. The CANCEL 
     * attempts to force a non-2xx response to the INVITE, therefore if a UAC 
     * wishes to give up on its call attempt entirely it can send a CANCEL.
     * <li>Stateful proxies - A stateful proxy may generate CANCEL requests 
     * for:
     * <ul>
     * <li>INVITE Requests - A CANCEL can be sent on pending INVITE client
     * transactions based on the period specified in the INVITE's Expires
     * header field elapsing.  However, this is generally unnecessary since
     * the endpoints involved will take care of signaling the end of the
     * transaction.
     * <li> Other Requests - A CANCEL can be sent on any other request the proxy
     * has generated at any time, subject to receiving a provisional response
     * to that request.
     * </ul>
     * </ul> 
     *
     * @param request the dialog creating request, the dialog of which this 
     * CANCEL will terminate.
     */
    public Request createCancelRequest(Request request);
    
    
// Standard Response Creation methods
   
    /**
     * Creates a new Response message of type specified by the statusCode 
     * paramater, containing the mandatory headers of the message with a body 
     * in the form of a Java object and the body content type.
     *
     * @param statusCode the new integer of the statusCode value of this Message.
     * @param callId the new CallIdHeader object of the callId value of this Message.
     * @param cSeq the new CSeqHeader object of the cSeq value of this Message.
     * @param from the new FromHeader object of the from value of this Message.
     * @param to the new ToHeader object of the to value of this Message.
     * @param via the new List object of the ViaHeaders of this Message.
     * @param contentType the new ContentTypeHeader object of the content type 
     * value of this Message.
     * @param content the new Object of the body content value of this Message.
     * @throws ParseException which signals that an error has been reached
     * unexpectedly while parsing the statusCode or the body.
     */
    public Response createResponse(int statusCode, CallIdHeader callId, 
            CSeqHeader cSeq, FromHeader from, ToHeader to, List via, 
                MaxForwardsHeader maxForwards, ContentTypeHeader contentType,
                Object content) throws ParseException;    
    
    /**
     * Creates a new Response message of type specified by the statusCode 
     * paramater, containing the mandatory headers of the message with a body 
     * in the form of a byte array and the body content type.
     *
     * @param statusCode the new integer of the statusCode value of this Message.
     * @param callId the new CallIdHeader object of the callId value of this Message.
     * @param cSeq the new CSeqHeader object of the cSeq value of this Message.
     * @param from the new FromHeader object of the from value of this Message.
     * @param to the new ToHeader object of the to value of this Message.
     * @param via the new List object of the ViaHeaders of this Message.
     * @param contentType the new ContentTypeHeader object of the content type 
     * value of this Message.
     * @param content the new byte array of the body content value of this Message.
     * @throws ParseException which signals that an error has been reached
     * unexpectedly while parsing the statusCode or the body.
     */
    public Response createResponse(int statusCode, CallIdHeader callId, 
            CSeqHeader cSeq, FromHeader from, ToHeader to, List via, 
                MaxForwardsHeader maxForwards, ContentTypeHeader contentType, 
                byte[] content) throws ParseException;   
    
    
    /**
     * Creates a new Response message of type specified by the statusCode 
     * paramater, containing the mandatory headers of the message. This new 
     * Response does not contain a body.
     *
     * @param statusCode the new integer of the statusCode value of this Message.
     * @param callId the new CallIdHeader object of the callId value of this Message.
     * @param cSeq the new CSeqHeader object of the cSeq value of this Message.
     * @param from the new FromHeader object of the from value of this Message.
     * @param to the new ToHeader object of the to value of this Message.
     * @param via the new List object of the ViaHeaders of this Message.
     * @throws ParseException which signals that an error has been reached
     * unexpectedly while parsing the statusCode.
     */
    public Response createResponse(int statusCode, CallIdHeader callId, 
            CSeqHeader cSeq, FromHeader from, ToHeader to, List via, 
                MaxForwardsHeader maxForwards) throws ParseException;  

// Response Creation methods based on a Request    

    /**
     * Creates a new Response message of type specified by the statusCode 
     * paramater, based on a specific Request with a new body in the form of a 
     * Java object and the body content type.
     *
     * @param statusCode the new integer of the statusCode value of this Message.
     * @param request the received Reqest object upon which to base the Response.
     * @param contentType the new ContentTypeHeader object of the content type 
     * value of this Message.
     * @param content the new Object of the body content value of this Message.
     * @throws ParseException which signals that an error has been reached
     * unexpectedly while parsing the statusCode or the body.
     */
    public Response createResponse(int statusCode, Request request, 
            ContentTypeHeader contentType, Object content) throws ParseException;    
    
    /**
     * Creates a new Response message of type specified by the statusCode 
     * paramater, based on a specific Request with a new body in the form of a 
     * byte array and the body content type.
     *
     * @param statusCode the new integer of the statusCode value of this Message.
     * @param request the received Reqest object upon which to base the Response.
     * @param contentType the new ContentTypeHeader object of the content type 
     * value of this Message.
     * @param content the new byte array of the body content value of this Message.
     * @throws ParseException which signals that an error has been reached
     * unexpectedly while parsing the statusCode or the body.
     */
    public Response createResponse(int statusCode, Request request, 
            ContentTypeHeader contentType, byte[] content) throws ParseException;

    /**
     * Creates a new Response message of type specified by the statusCode 
     * paramater, based on a specific Request message. This new Response does 
     * not contain a body.
     *
     * @param statusCode the new integer of the statusCode value of this Message.
     * @param request the received Reqest object upon which to base the Response.
     * @throws ParseException which signals that an error has been reached
     * unexpectedly while parsing the statusCode.
     */
    public Response createResponse(int statusCode, Request request) 
                                                throws ParseException;    
    
}

